#
# Copyright (c) 2015, Red Hat, Inc.
#
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
#
# 1. Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in the
#    documentation and/or other materials provided with the
#    distribution.
# 3. Neither the name of the Red Hat nor the names of its
#    contributors may be used to endorse or promote products derived
#    from this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
# A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
# OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
# LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
# DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
# THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#
# Authors:  Michael Simacek <msimacek@redhat.com>

from __future__ import print_function

import os
import socket

from javapackages.common.exception import JavaPackagesToolsException
from javapackages.maven.artifact import Artifact

socket_path = os.environ.get('PM_REQUEST_SOCKET')

class InstallationException(JavaPackagesToolsException):
    def __init__(self, dep, out):
        super(InstallationException, self).__init__()
        self.dep = dep
        self.out = out

class ConnectionException(JavaPackagesToolsException):
    pass

def install_artifact(dep):
    if not socket_path:
        raise ConnectionException("PM_REQUEST_SOCKET environment variable not set")
    sock = socket.socket(socket.AF_UNIX, socket.SOCK_STREAM)
    try:
        sock.connect(socket_path)
        sock.sendall("install '{0}'\n".format(dep).encode())
        fo = sock.makefile()
        status = fo.readline()
        out = fo.read()
        if status != 'ok\n':
            raise InstallationException(dep, out)
    except socket.error as e:
        raise ConnectionException(str(e))
    finally:
        sock.close()

def install_maven_artifact(artifact_spec):
    artifact = Artifact.from_mvn_str(artifact_spec)
    try:
        install_artifact(artifact.get_rpm_str(compat_ver=artifact.version))
    except InstallationException:
        if not artifact.version:
            raise
        install_artifact(artifact.get_rpm_str())



import os
import unittest
from shutil import rmtree
import lxml.etree as ET

from javapackages.maven.artifact import Artifact
from javapackages.xmvn.xmvn_config import XMvnConfig


class TestXMvnConfig(unittest.TestCase):

    def __clean(self):
        rmtree(".xmvn", ignore_errors=True)

    def __find(self, elem, xpath):
        ret = None
        ret = elem.xpath(xpath, namespaces=dict(xmvn=XMvnConfig.XMLNS))
        if not ret:
            ret = elem.xpath(xpath.replace('xmvn:',XMvnConfig.XMLNS))[0]
        else:
            ret = ret[0]
        return ret

    def __findall(self, elem, xpath):
        ret = elem.xpath(xpath, namespaces=dict(xmvn=XMvnConfig.XMLNS))
        if not ret:
           ret = elem.xpath(xpath.replace('xmvn:',XMvnConfig.XMLNS))
        return ret

    def _read_current_conf(self):
        ind = self._read_index()

        fname = 'javapackages-config-{index:05d}.xml'.format(index=ind)
        confpath = os.path.join(".xmvn", "config.d", fname)
        parser = ET.XMLParser(remove_blank_text=True)
        et = ET.parse(confpath, parser)
        return et


    def _read_index(self):
        with open(".xmvn/javapackages-rule-index") as jin:
            return int(jin.read())

    def setUp(self):
        self.origDir = os.getcwd()
        os.mkdir("TestXMvnConfig-run")
        os.chdir("TestXMvnConfig-run")

    def tearDown(self):
        os.chdir(self.origDir)
        rmtree("TestXMvnConfig-run")

    def test_index_init(self):
        self.__clean()
        xc = XMvnConfig()
        a = Artifact('gId', 'aId')
        a1 = Artifact('agId', 'aaId')
        xc.add_aliases(a, [a1])
        ind = self._read_index()
        self.assertEqual(ind, 1)

        xc.add_aliases(a, [a1])
        ind = self._read_index()
        self.assertEqual(ind, 2)

    def test_aliases(self):
        xc = XMvnConfig()
        a = Artifact('gId', 'aId')
        a1 = Artifact('gIda1', 'aIda1')
        a2 = Artifact('gIda2', 'aIda2')
        xc.add_aliases(a, [a1, a2])

        et = self._read_current_conf()
        root = et.getroot()
        rule = self.__find(root, "xmvn:artifactManagement/xmvn:rule")

        aglob = self.__find(rule, "xmvn:artifactGlob")
        self.assertEqual(self.__find(aglob, "xmvn:groupId").text.strip(), "gId")
        self.assertEqual(self.__find(aglob,
                                     "xmvn:artifactId").text.strip(), "aId")

        aliases = self.__findall(rule, "xmvn:aliases/xmvn:alias")
        self.assertEqual(self.__find(aliases[0],
                                     "xmvn:groupId").text.strip(), "gIda1")
        self.assertEqual(self.__find(aliases[0],
                                     "xmvn:artifactId").text.strip(), "aIda1")

        self.assertEqual(self.__find(aliases[1], "xmvn:groupId").text.strip(), "gIda2")
        self.assertEqual(self.__find(aliases[1], "xmvn:artifactId").text.strip(), "aIda2")

    def test_file_mappings(self):
        xc = XMvnConfig()
        a = Artifact('gId', 'aId')
        xc.add_file_mapping(a, ['path1','path2'])

        et = self._read_current_conf()
        root = et.getroot()
        rule = self.__find(root, "xmvn:artifactManagement/xmvn:rule")

        aglob = self.__find(rule, "xmvn:artifactGlob")
        self.assertEqual(self.__find(aglob, "xmvn:groupId").text.strip(), "gId")
        self.assertEqual(self.__find(aglob, "xmvn:artifactId").text.strip(), "aId")

        files = self.__findall(rule, "xmvn:files/xmvn:file")
        self.assertEqual(files[0].text.strip(), "path1")
        self.assertEqual(files[1].text.strip(), "path2")

    def test_compat_versions(self):
        xc = XMvnConfig()
        a = Artifact('gId', 'aId')
        xc.add_compat_versions(a, ['version1','version2'])

        et = self._read_current_conf()
        root = et.getroot()
        rule = self.__find(root, "xmvn:artifactManagement/xmvn:rule")

        aglob = self.__find(rule, "xmvn:artifactGlob")
        self.assertEqual(self.__find(aglob, "xmvn:groupId").text.strip(), "gId")
        self.assertEqual(self.__find(aglob, "xmvn:artifactId").text.strip(), "aId")

        files = self.__findall(rule, "xmvn:versions/xmvn:version")
        self.assertEqual(files[0].text.strip(), "version1")
        self.assertEqual(files[1].text.strip(), "version2")

    def test_package_mappings(self):
        xc = XMvnConfig()
        a = Artifact('gId', 'aId')
        xc.add_package_mapping(a, 'package')

        et = self._read_current_conf()
        root = et.getroot()
        rule = self.__find(root, "xmvn:artifactManagement/xmvn:rule")

        aglob = self.__find(rule, "xmvn:artifactGlob")
        self.assertEqual(self.__find(aglob, "xmvn:groupId").text.strip(), "gId")
        self.assertEqual(self.__find(aglob, "xmvn:artifactId").text.strip(), "aId")

        self.assertEqual(self.__find(rule, "xmvn:targetPackage").text.strip(),
                         "package")

    def test_custom_config(self):
        xc = XMvnConfig()
        xc.add_custom_option("buildSettings/compilerSource", "1.5")
        et = self._read_current_conf()
        root = et.getroot()
        cs = self.__find(root, "xmvn:buildSettings/xmvn:compilerSource")
        self.assertEqual(cs.text.strip(), "1.5")

    def test_custom_config_xml(self):
        xc = XMvnConfig()
        xc.add_custom_option("buildSettings/compilerSource",
                """<versions>
                    <version>1.5</version>
                    <version>1.6</version>
                </versions>
                """)
        et = self._read_current_conf()
        root = et.getroot()
        versions = self.__find(root,
                "xmvn:buildSettings/xmvn:compilerSource/xmvn:versions")
        self.assertEqual(len(versions), 2)
        v1, v2 = self.__findall(versions, "xmvn:version")
        self.assertEqual(v1.text.strip(), "1.5")
        self.assertEqual(v2.text.strip(), "1.6")


if __name__ == '__main__':
    unittest.main()



#
# Copyright (c) 2013, Red Hat, Inc
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
#
# 1. Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in the
#    documentation and/or other materials provided with the
#    distribution.
# 3. Neither the name of Red Hat nor the names of its
#    contributors may be used to endorse or promote products derived
#    from this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
# A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
# OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
# LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
# DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
# THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#
# Authors:  Stanislav Ochotnicky <sochotnicky@redhat.com>
"""
Module for handling, manipulating and querying various files related to Java
packaging in Linux distributions.
"""


try:
    from javapackages.version import __version__
except ImportError:
    import os
    dirpath = os.path.dirname(os.path.realpath(__file__))
    versionpath = os.path.join(dirpath, '..', '..', 'VERSION')
    with open(versionpath) as vp:
        __version__ = vp.read().strip()



from test_common import exec_pom_macro_simple, WorkdirTestCase
import unittest

class TestPomArtifact(WorkdirTestCase):
    def test_gid1(self):
        return_value, stderr, report = exec_pom_macro_simple(\
                "%pom_remove_dep commons-ioX",
                'example_common.xml', 'example_common_removed_commons_iox.xml')
        self.assertEqual(0, return_value, stderr)
        self.assertEqual('', report, report)

    def test_gid2(self):
        return_value, stderr, report = exec_pom_macro_simple(\
                "%pom_remove_dep commons-ioX:",
                'example_common.xml', 'example_common_removed_commons_iox.xml')
        self.assertEqual(0, return_value, stderr)
        self.assertEqual('', report, report)

    def test_aid(self):
        return_value, stderr, report = exec_pom_macro_simple(\
                "%pom_remove_dep :junit",
                'example_schemas.xml', 'example_schemas_removed_junit.xml')
        self.assertEqual(0, return_value, stderr)
        self.assertEqual('', report, report)

    def test_all(self):
        return_value, stderr, report = exec_pom_macro_simple(\
                "%pom_remove_dep :",
                'example_common.xml', 'example_common_removed_all_deps.xml')
        self.assertEqual(0, return_value, stderr)
        self.assertEqual('', report, report)

    def test_version(self):
        return_value, stderr, report = exec_pom_macro_simple(\
                "%pom_add_dep org.apache.uima:uimaj:2.5.0",
                'minimal_pom.xml', 'minimal_pom_add_uimaj.xml')
        self.assertEqual(0, return_value, stderr)
        self.assertEqual('', report, report)

    def test_scope(self):
        return_value, stderr, report = exec_pom_macro_simple(\
                "%pom_add_dep org.apache.ant:ant-parent::test",
                'minimal_pom.xml', 'minimal_pom_add_ant.xml')
        self.assertEqual(0, return_value, stderr)
        self.assertEqual('', report, report)

    def test_classifier(self):
        return_value, stderr, report = exec_pom_macro_simple(\
                "%pom_add_dep org.apache.ant:ant-parent:::cls",
                'minimal_pom.xml', 'minimal_pom_add_cls.xml')
        self.assertEqual(0, return_value, stderr)
        self.assertEqual('', report, report)

class TestPomChange(WorkdirTestCase):
    def test_change_simple(self):
        return_value, stderr, report = exec_pom_macro_simple(\
                "%pom_change_dep junit:junit junit:junit3",
                'example_common.xml', 'example_common_junit3.xml')
        self.assertEqual(0, return_value, stderr)
        self.assertEqual('', report, report)

    def test_copy_aid1(self):
        return_value, stderr, report = exec_pom_macro_simple(\
                "%pom_change_dep commons-io org.apache.commons-io",
                'example_common.xml', 'example_common_apache.xml')
        self.assertEqual(0, return_value, stderr)
        self.assertEqual('', report, report)

    def test_copy_aid2(self):
        return_value, stderr, report = exec_pom_macro_simple(\
                "%pom_change_dep commons-io: org.apache.commons-io:",
                'example_common.xml', 'example_common_apache.xml')
        self.assertEqual(0, return_value, stderr)
        self.assertEqual('', report, report)

    def test_copy_gid(self):
        return_value, stderr, report = exec_pom_macro_simple(\
                "%pom_change_dep :commons-io :commons",
                'example_common.xml', 'example_common_commons.xml')
        self.assertEqual(0, return_value, stderr)
        self.assertEqual('', report, report)

    def test_remove_scope(self):
        return_value, stderr, report = exec_pom_macro_simple(\
                "%pom_change_dep :::test :::-",
                'simple_pom.xml', 'simple_removed_scope.xml')
        self.assertEqual(0, return_value, stderr)
        self.assertEqual('', report, report)

    def test_keep_classifier(self):
        return_value, stderr, report = exec_pom_macro_simple(\
                "%pom_change_dep commons-io org.apache.commons.commons-io:",
                'classifier.xml', 'keep_classifier.xml')
        self.assertEqual(0, return_value, stderr)
        self.assertEqual('', report, report)

    def test_keep_optional(self):
        return_value, stderr, report = exec_pom_macro_simple(\
                "%pom_change_dep commons-io: org.apache.commons-io:",
                'optional.xml', 'keep_optional.xml')
        self.assertEqual(0, return_value, stderr)
        self.assertEqual('', report, report)

if __name__ == '__main__':
    unittest.main()



from test_common import exec_pom_macro, WorkdirTestCase, assertIn
import unittest

example_tree = {
        '': 'example_parent.xml',
        'common': 'example_common.xml',
        'intermediate': 'example_intermediate.xml',
        'intermediate/schemas': 'example_schemas.xml'}

class TestPomPath(WorkdirTestCase):
    def test_sanity(self):
        return_value, stderr, report = exec_pom_macro('', {
            'submodule': 'minimal_pom.xml'})
        self.assertEqual(0, return_value, stderr)
        self.assertEqual('', report, report)

    def test_default_nonexistent(self):
        return_value, stderr, _ = exec_pom_macro('%pom_remove_parent', {})
        assertIn(self, "Couldn't locate ", stderr)
        self.assertNotEqual(0, return_value)

    def test_submodule_nonexistent(self):
        return_value, stderr, _ = exec_pom_macro('%pom_remove_parent submodule', {})
        assertIn(self, "Couldn't locate ", stderr)
        self.assertNotEqual(0, return_value)

    def test_default(self):
        return_value, stderr, report = exec_pom_macro('%pom_remove_parent',
                poms_tree={'': 'minimal_pom.xml'},
                want_tree={'': 'minimal_removed_parent.xml'})
        self.assertEqual(0, return_value, stderr)
        self.assertEqual('', report, report)

    def test_explicit(self):
        return_value, stderr, report = exec_pom_macro('%pom_remove_parent pom.xml',
                poms_tree={'': 'minimal_pom.xml'},
                want_tree={'': 'minimal_removed_parent.xml'})
        self.assertEqual(0, return_value, stderr)
        self.assertEqual('', report, report)

    def test_submodule_default(self):
        return_value, stderr, report = exec_pom_macro('%pom_remove_parent submodule',
                poms_tree={'submodule': 'minimal_pom.xml'},
                want_tree={'submodule': 'minimal_removed_parent.xml'})
        self.assertEqual(0, return_value, stderr)
        self.assertEqual('', report, report)

    def test_submodule_explicit(self):
        return_value, stderr, report = exec_pom_macro('%pom_remove_parent submodule',
                poms_tree={'submodule': 'minimal_pom.xml'},
                want_tree={'submodule': 'minimal_removed_parent.xml'})
        self.assertEqual(0, return_value, stderr)
        self.assertEqual('', report, report)

    def test_submodule_more_levels(self):
        return_value, stderr, report = exec_pom_macro('%pom_remove_parent submodule/something',
                poms_tree={'submodule/something': 'minimal_pom.xml'},
                want_tree={'submodule/something': 'minimal_removed_parent.xml'})
        self.assertEqual(0, return_value, stderr)
        self.assertEqual('', report, report)

class TestMorePoms(WorkdirTestCase):
    def test_submodules_default(self):
        return_value, stderr, report = exec_pom_macro('%pom_remove_parent submodule1 submodule2',
                poms_tree={'submodule1': 'minimal_pom.xml',
                    'submodule2': 'simple_pom.xml'},
                want_tree={'submodule1': 'minimal_removed_parent.xml',
                    'submodule2': 'simple_removed_parent.xml'})
        self.assertEqual(0, return_value, stderr)
        self.assertEqual('', report, report)

    def test_submodules_different_levels(self):
        return_value, stderr, report = exec_pom_macro('%pom_remove_parent . submodule2',
                poms_tree={'submodule1': 'minimal_pom.xml',
                    'submodule2': 'simple_pom.xml',
                    '': 'minimal_pom.xml'},
                want_tree={'submodule2': 'simple_removed_parent.xml',
                    '': 'minimal_removed_parent.xml'})
        self.assertEqual(0, return_value, stderr)
        self.assertEqual('', report, report)

    def test_many_submodules(self):
        poms = ['submodule{0}'.format(i) for i in range(10)]
        command = '%pom_remove_parent ' + ' '.join(poms)
        poms_tree = {'untouched': 'minimal_pom.xml'}
        want_tree = {}
        for pom in poms:
            poms_tree[pom] = 'simple_pom.xml'
            want_tree[pom] = 'simple_removed_parent.xml'
        return_value, stderr, report = exec_pom_macro(command, poms_tree, want_tree)
        self.assertEqual(0, return_value, stderr)
        self.assertEqual('', report, report)

    def test_submodules_partial_match(self):
        return_value, stderr, _ = exec_pom_macro(\
                '%pom_remove_dep :commons-io submodule1 submodule2',
                poms_tree={'submodule1': 'minimal_pom.xml',
                    'submodule2': 'simple_pom.xml'})
        assertIn(self, "Error", stderr)
        assertIn(self, "not found", stderr)
        self.assertNotEqual(0, return_value)

    def test_submodules_partial_match_force(self):
        return_value, stderr, report = exec_pom_macro(\
                '%pom_remove_dep -f :commons-io submodule1 submodule2',
                poms_tree={'submodule1': 'minimal_pom.xml',
                    'submodule2': 'simple_pom.xml'},
                want_tree={'submodule2': 'simple_removed_commons_io.xml'})
        self.assertEqual(0, return_value, stderr)
        self.assertEqual('', report, report)

class TestRecursive(WorkdirTestCase):
    def test_default_recursive1(self):
        return_value, stderr, report = exec_pom_macro('%pom_remove_dep -r :commons-io',
                poms_tree=example_tree,
                want_tree={'intermediate': 'example_intermediate_removed_commons_io.xml',
                    'common': 'example_common_removed_commons_io.xml'})
        self.assertEqual(0, return_value, stderr)
        self.assertEqual('', report, report)

    def test_default_recursive2(self):
        return_value, stderr, report = exec_pom_macro('%pom_remove_dep -r :junit',
                poms_tree=example_tree,
                want_tree={'intermediate/schemas': 'example_schemas_removed_junit.xml',
                    'common': 'example_common_removed_junit.xml'})
        self.assertEqual(0, return_value, stderr)
        self.assertEqual('', report, report)

    def test_recursive_plugin_with_module(self):
        poms_tree = dict(example_tree)
        poms_tree[''] = 'example_parent_plugin_with_module.xml'
        return_value, stderr, report = exec_pom_macro('%pom_remove_dep -r :junit',
                poms_tree=poms_tree,
                want_tree={'intermediate/schemas': 'example_schemas_removed_junit.xml'})
        self.assertEqual(0, return_value, stderr)
        self.assertEqual('', report, report)

    def test_default_recursive_profile(self):
        poms_tree = dict(example_tree)
        poms_tree[''] = 'example_parent_profile.xml'
        return_value, stderr, report = exec_pom_macro('%pom_remove_dep -r :commons-io',
                poms_tree=poms_tree,
                want_tree={'intermediate': 'example_intermediate_removed_commons_io.xml',
                    'common': 'example_common_removed_commons_io.xml'})
        self.assertEqual(0, return_value, stderr)
        self.assertEqual('', report, report)

    def test_recursive_no_match(self):
        return_value, stderr, _ = exec_pom_macro('%pom_remove_dep -r :not-there',
                poms_tree=example_tree)
        assertIn(self, "Error", stderr)
        assertIn(self, "not found", stderr)
        self.assertNotEqual(0, return_value)

    def test_recursive_missing(self):
        poms_tree = dict(example_tree)
        del poms_tree['intermediate/schemas']
        return_value, stderr, _ = exec_pom_macro('%pom_remove_dep -r :commons-io',
                poms_tree=poms_tree)
        assertIn(self, "Couldn't locate", stderr)
        self.assertNotEqual(0, return_value)

    def test_recursive_subpom(self):
        return_value, stderr, report = exec_pom_macro('%pom_remove_dep -r :junit intermediate',
                poms_tree=example_tree,
                want_tree={'intermediate/schemas': 'example_schemas_removed_junit.xml'})
        self.assertEqual(0, return_value, stderr)
        self.assertEqual('', report, report)

    def test_recursive_multiple(self):
        poms_tree = dict(example_tree)
        return_value, stderr, report = exec_pom_macro('%pom_remove_dep -r :junit intermediate common',
                poms_tree=poms_tree,
                want_tree={'intermediate/schemas': 'example_schemas_removed_junit.xml',
                    'common': 'example_common_removed_junit.xml'})
        self.assertEqual(0, return_value, stderr)
        self.assertEqual('', report, report)

    def test_recursive_partial_match(self):
        return_value, stderr, _ = exec_pom_macro('%pom_remove_dep -r commons-ioX intermediate common',
                poms_tree=example_tree)
        assertIn(self, "Error", stderr)
        assertIn(self, "not found", stderr)
        self.assertNotEqual(0, return_value)

    def test_recursive_partial_match_forced(self):
        return_value, stderr, report = exec_pom_macro(\
                '%pom_remove_dep -rf commons-ioX:commons-io intermediate common',
                poms_tree=example_tree,
                want_tree={'common': 'example_common_removed_commons_iox.xml'})
        self.assertEqual(0, return_value, stderr)
        self.assertEqual('', report, report)

class TestAttributeRecognition(WorkdirTestCase):
    def test_default_extra_xml(self):
        return_value, stderr, report = exec_pom_macro('\
                %pom_add_dep org.apache.uimaj:uimaj-core "<scope>test</scope>"',
                poms_tree={'': 'minimal_pom.xml'},
                want_tree={'': 'minimal_add_dep_scope.xml'})
        self.assertEqual(0, return_value, stderr)
        self.assertEqual('', report, report)

    def test_explicit_extra_xml(self):
        return_value, stderr, report = exec_pom_macro('\
                %pom_add_dep org.apache.uimaj:uimaj-core pom.xml "<scope>test</scope>"',
                poms_tree={'': 'minimal_pom.xml'},
                want_tree={'': 'minimal_add_dep_scope.xml'})
        self.assertEqual(0, return_value, stderr)
        self.assertEqual('', report, report)

    def test_submodule_extra_xml(self):
        return_value, stderr, report = exec_pom_macro('\
                %pom_add_dep org.apache.uimaj:uimaj-core submodule "<scope>test</scope>"',
                poms_tree={'submodule': 'minimal_pom.xml'},
                want_tree={'submodule': 'minimal_add_dep_scope.xml'})
        self.assertEqual(0, return_value, stderr)
        self.assertEqual('', report, report)

    def test_submodules_extra_xml(self):
        return_value, stderr, report = exec_pom_macro(\
                '%pom_add_dep org.apache.uimaj:uimaj-core submodule1 submodule2 "<scope>test</scope>"',
                poms_tree={'submodule1': 'minimal_pom.xml',
                    'submodule2': 'minimal_pom.xml'},
                want_tree={'submodule1': 'minimal_add_dep_scope.xml',
                    'submodule2': 'minimal_add_dep_scope.xml'})
        self.assertEqual(0, return_value, stderr)
        self.assertEqual('', report, report)

    def test_dep_mgmt_extra_xml(self):
        return_value, stderr, report = exec_pom_macro(\
                '%pom_add_dep_mgmt org.apache.uimaj:uimaj-core submodule1 submodule2 "<scope>test</scope>"',
                poms_tree={'submodule1': 'minimal_pom.xml',
                    'submodule2': 'minimal_pom.xml'},
                want_tree={'submodule1': 'minimal_add_dep_mgmt_scope.xml',
                    'submodule2': 'minimal_add_dep_mgmt_scope.xml'})
        self.assertEqual(0, return_value, stderr)
        self.assertEqual('', report, report)

    def test_plugin_extra_xml(self):
        return_value, stderr, report = exec_pom_macro(\
                '%pom_add_plugin org.apache.maven:maven-remote-resources-plugin \
                submodule1 submodule2 "<goal>generate-resources</goal>"',
                poms_tree={'submodule1': 'minimal_pom.xml',
                    'submodule2': 'minimal_pom.xml'},
                want_tree={'submodule1': 'minimal_add_plugin_resource.xml',
                    'submodule2': 'minimal_add_plugin_resource.xml'})
        self.assertEqual(0, return_value, stderr)
        self.assertEqual('', report, report)

if __name__ == '__main__':
    unittest.main()



#
# Copyright (c) 2015, Red Hat, Inc.
#
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
#
# 1. Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in the
#    documentation and/or other materials provided with the
#    distribution.
# 3. Neither the name of the Red Hat nor the names of its
#    contributors may be used to endorse or promote products derived
#    from this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
# A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
# OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
# LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
# DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
# THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#
# Authors:  Stanislav Ochotnicky <sochotnicky@redhat.com>
#           Michal Srb <msrb@redhat.com>

import javapackages.maven.pomreader as POMReader
from javapackages.maven.pomreader import PomLoadingException
from javapackages.maven.dependency import Dependency
from javapackages.maven.plugin import Plugin
from javapackages.maven.extension import Extension

import os


class POM(object):
    """
    Class for querying basic information from pom.xml files
    """
    def __init__(self, path):
        if not path:
            raise PomLoadingException("Path \"{p}\" is invalid".format(p=path))
        self._doc = POMReader.load(path)
        self._path = os.path.join(path)

    def __str__(self):
        return ":".join([self.groupId, self.artifactId, self.version])

    def has_parent(self):
        parent = POMReader.find(self._doc, "./pom:parent")
        if parent is not None:
            return True
        return False

    @property
    def parent(self):
        aId = POMReader.find(self._doc, "./pom:parent/pom:artifactId")
        if aId is None:
            return None
        artifactId = aId.text

        groupId = ""
        gId = POMReader.find(self._doc, "./pom:parent/pom:groupId")
        if gId is not None:
            groupId = gId.text

        version = ""
        ver = POMReader.find(self._doc, "./pom:parent/pom:version")
        if ver is not None:
            version = ver.text

        relativePath = ""
        relPath = POMReader.find(self._doc, "./pom:parent/pom:relativePath")
        if relPath is not None:
            relativePath = relPath.text

        return ParentPOM(groupId, artifactId, version, relativePath)

    @property
    def groupId(self):
        """
        Effective groupId of the pom artifact taking into account parent groupId
        """
        gId = POMReader.find(self._doc, "./pom:groupId")
        if gId is None:
            gId = POMReader.find(self._doc, "./pom:parent/pom:groupId")
        if gId is None:
            raise PomLoadingException("Unable to determine groupId")
        return gId.text.strip()

    @property
    def artifactId(self):
        """
        Effective artifactId of the pom artifact
        """
        aId = POMReader.find(self._doc, "./pom:artifactId")
        if aId is None:
            raise PomLoadingException("Unable to determine artifactId")
        return aId.text.strip()

    @property
    def version(self):
        """
        Effective version of the pom artifact taking into account parent
        version
        """
        version = POMReader.find(self._doc, "./pom:version")
        if version is None:
            version = POMReader.find(self._doc, "./pom:parent/pom:version")
        if version is None:
            raise PomLoadingException("Unable to determine artifact version")
        return version.text.strip()

    @property
    def packaging(self):
        """
        Packaging type of artifact or "jar" if unspecified
        """
        packaging = POMReader.find(self._doc, "./pom:packaging")
        if packaging is None:
            # use default packaging type
            return "jar"
        return packaging.text.strip()

    @property
    def dependencies(self):
        """
        List of dependencies
        """
        xmlnodes = POMReader.xpath(self._doc,
                                   "./pom:dependencies/pom:dependency")
        return [Dependency.from_xml_element(x) for x in xmlnodes]

    @property
    def dependencyManagement(self):
        """
        List of dependencies from dependency management section
        """
        xmlnodes = POMReader.xpath(self._doc,
                                   "./pom:dependencyManagement"
                                   "/pom:dependencies/pom:dependency")
        return [Dependency.from_xml_element(x) for x in xmlnodes]

    @property
    def pluginManagement(self):
        """
        List of plugins from plugin management section
        """
        xmlnodes = POMReader.xpath(self._doc,
                                   "./pom:pluginManagement"
                                   "/pom:plugins/pom:plugin")
        return [Plugin.from_xml_element(x) for x in xmlnodes]

    @property
    def plugins(self):
        """
        List of plugins
        """
        xmlnodes = POMReader.xpath(self._doc,
                                   "./pom:build/pom:plugins/pom:plugin")
        return [Plugin.from_xml_element(x) for x in xmlnodes]

    @property
    def extensions(self):
        """
        List of extensions
        """
        xmlnodes = POMReader.xpath(self._doc,
                                   "./pom:build/pom:extensions/pom:extension")
        return [Extension.from_xml_element(x) for x in xmlnodes]

    @property
    def properties(self):
        """
        Dictionary consisting of properties specified in pom.xml
        """
        properties = {}
        xmlnodes = POMReader.find(self._doc, "./pom:properties")
        if xmlnodes is None:
            return properties
        propnodes = xmlnodes.getchildren()
        for node in propnodes:
            if node.tag.startswith('{'):
                tag = node.tag[node.tag.find('}') + 1:]
            else:
                tag = node.tag
            properties[tag] = node.text

        return properties


class ParentPOM(object):
    def __init__(self, groupId, artifactId, version="", relativePath=""):
        self.groupId = ""
        self.artifactId = ""

        if groupId is not None:
            self.groupId = groupId.strip()
        if artifactId is not None:
            self.artifactId = artifactId.strip()
        if version is not None:
            self.version = version.strip()
        if relativePath is not None:
            self.relativePath = relativePath.strip()



from lxml import etree
import sys
import re

def compare_xml_files(file1, file2, unordered=None):
    """ Compares two XML files and returns empty string if they're identical
    and string with differences otherwise. Optional third argument specifies
    which elements should be compared without taking element order into
    account. It is represented by list of xpath-like epressions with * as a
    wildcard.
    Examples: ['*'] for unordered comparison of all elements.
              ['/html/head'] for unordered comparison of children of <head>
                    element under the root <html>
              ['p'] for unordered comparison of children of every <p> element
    """
    return compare_lxml_etree(etree.parse(file1).getroot(),
            etree.parse(file2).getroot(), unordered)

def compare_lxml_etree(doc1, doc2, unordered=None):
    """ Compares two XML documents (lxml.etree) and returns empty string if
    they're identical and string with differences otherwise. Optional third
    argument specifies which elements should be compared without taking element
    order into account. It is represented by list of xpath-like epressions with
    * as a wildcard.
    Examples: ['*'] for unordered comparison of all elements.
              ['/html/head'] for unordered comparison of children of <head>
                    element under the root <html>
              ['p'] for unordered comparison of children of every <p> element
    """

    paths = _create_paths(unordered)

    _strip_comments(doc1)
    _strip_comments(doc2)

    lines = []
    for sgn, line in _compare_element(doc1, doc2, unordered_paths=paths):
        lines.append('{sgn}{line}'.format(sgn=sgn, line=line))

    return '\n'.join(lines)

def _strip_comments(doc):
    for node in doc.iter():
        if node.tag == etree.Comment:
            parent = node.getparent()
            parent.remove(node)

def _localname(tag):
    return etree.QName(tag).localname

def _compare_element(exp, act, unordered_paths=None):
    if exp.tag != act.tag:
        return _print_element(exp, '-') + _print_element(act, '+')

    tag = _localname(exp.tag)
    new_unordered = _descend_paths(unordered_paths, tag)
    def callback(exp, act):
        return _compare_element(exp, act, unordered_paths=new_unordered)

    if _matches_paths(unordered_paths, tag):
        child_rep = _tab(_compare_children_unordered(exp, act, callback))
    else:
        child_rep = _tab(_compare_children_ordered(exp, act, callback))

    attr_rep = _tab(_compare_attrs(exp, act))
    text_rep = _tab(_compare_text(exp.text, act.text))
    tail_rep = _compare_text(exp.tail, act.tail)

    if attr_rep or child_rep or text_rep:
        rep = []
        end = ''
        if not attr_rep:
            end = '>' if child_rep or text_rep else '/>'

        rep.append((' ', '<{tag}{end}'.format(tag=tag, end=end)))

        if attr_rep:
            rep += attr_rep
            rep.append((' ', '>' if child_rep or text_rep else '/>'))

        if child_rep or text_rep:
            rep.extend(text_rep)
            rep.extend(child_rep)
            rep.append((' ', '</{tag}>'.format(tag=tag)))

        rep.extend(tail_rep)
        return rep
    else:
        if tail_rep:
            return [(' ', '<{tag} .../>'.format(tag=tag))] + tail_rep
        else: return []

class _Path(object):
    def __init__(self, parts, relative=False):
        self.parts = list(parts)
        self.relative = relative

    def matches_start(self, tagname):
        exp = re.escape(self.parts[0])
        exp = re.sub(r'\\\*', '.*', exp)
        return re.match(exp + '$', tagname)

    def matches(self, tagname):
        return len(self.parts) == 1 and self.matches_start(tagname)

    def descend(self):
        if len(self.parts) > 1:
            return _Path(self.parts[1:])

def _create_paths(paths):
    res = []
    for path in paths or []:
        if path.startswith('/'):
            parts = path.split('/')[1:]
            res.append(_Path(parts))
        else:
            parts = path.split('/')
            res.append(_Path(parts))
            res.append(_Path(parts, relative=True))
    return res

def _matches_paths(paths, tagname):
    for path in paths or []:
        if path.matches(tagname):
            return True
    return False

def _descend_paths(paths, tagname):
    res = []
    for path in paths or []:
        if path.relative:
            res.append(path)
            if path.matches_start(tagname):
                descendant = path.descend()
                if descendant:
                    res.append(descendant)
        else:
            descendant = path.descend()
            if descendant:
                res.append(descendant)
    return res

def _tab(lst):
    return [(sgn, '    ' + line) for sgn, line in lst]

def _print_element(elem, prefix):
    if elem is None:
        return
    rep = []
    attrs = ['{k}="{v}"'.format(k=k, v=v) for k, v in elem.attrib.iteritems()]
    tag = _localname(elem.tag)
    if len(elem) or elem.text:
        rep.append('<{elem}{attrs}>'.format(elem=tag,
            attrs=(' ' + ' '.join(attrs)) if attrs else ''))
        if elem.text and elem.text.strip():
            rep.append('    ' + elem.text.strip())
        rep += ['    ' + line for child in elem for
                _, line in _print_element(child, prefix)]
        rep.append('</{elem}>'.format(elem=tag))
    else:
        rep.append('<{elem}{attrs}/>'.format(elem=tag,
            attrs=(' ' + ' '.join(attrs)) if attrs else ''))
    return [(prefix, line) for line in rep]

def _compare_children_unordered(exp, act, cmp_f):
    r_matched = set()
    index = []

    for i, lchild in enumerate(exp):
        first = None
        for j, rchild in enumerate(act):
            if j not in r_matched:
                if lchild.tag == rchild.tag:
                    subrep = cmp_f(lchild, rchild)
                    if not subrep:
                        r_matched.add(j)
                        break
                    elif not first:
                        first = (j, subrep)
        else:
            if first:
                j, subrep = first
                r_matched.add(j)
                index.append((i, subrep))
            else:
                index.append((i, _print_element(lchild, '-')))

    for j, rchild in enumerate(act):
        if j not in r_matched:
            index.append((j, _print_element(rchild, '+')))

    return _get_items_from_index(index)

def _get_items_from_index(index):
    items = []
    for _, item in sorted(index, key=lambda tup: tup[0]):
        items += item
    return items

def _compare_children_ordered(exp, act, cmp_f):
    rep = []
    for i, lchild in enumerate(exp):
        if i < len(act):
            rep += cmp_f(lchild, act[i])
        else:
            rep += _print_element(lchild, '-')
    for i in range(len(exp), len(act)):
        rep += _print_element(act[i], '+')
    return rep

def _compare_text(text1, text2):
    text1 = text1.strip() if text1 else None
    text2 = text2.strip() if text2 else None
    rep = []
    if text1 != text2:
        if text1:
            rep.append(('-', text1))
        if text2:
            rep.append(('+', text2))
    return rep

def _compare_attrs(exp, act):
    rep = []
    attrib1, attrib2 = exp.attrib, act.attrib

    for k, lchild in attrib1.iteritems():
        if k in attrib2:
            if attrib2[k] != lchild:
                rep.append(('-', '{name} = "{value}"'\
                        .format(name=k, value=lchild)))
                rep.append(('+', '{name} = "{value}"'\
                        .format(name=k, value=attrib2[k])))
        else:
            rep.append(('-', '{name} = "{value}"'.format(name=k, value=lchild)))

    for k, lchild in attrib2.iteritems():
        if k not in attrib1:
            rep.append(('+', '{name} = "{value}"'.format(name=k, value=lchild)))

    return rep

if __name__ == '__main__':
    if len(sys.argv) < 3:
        print('Usage {name} filexp filact ' +
            '[unordered_elements path specification]').format(name=sys.argv[0])
        sys.exit(1)
    print(compare_xml_files(sys.argv[1], sys.argv[2], sys.argv[3:]))



from test_common import exec_pom_macro, WorkdirTestCase, assertIn
from pom_editor_paths_test import example_tree
import unittest

class TestPomXpathDisable(WorkdirTestCase):
    def test_no_match(self):
        return_value, stderr, _ = exec_pom_macro(
            '''%pom_xpath_disable "/pom:project/pom:groupId='not-there'" submodule''', {
            'submodule': 'minimal_pom.xml'})
        assertIn(self, "Error", stderr)
        assertIn(self, "didn't match", stderr)
        self.assertNotEqual(0, return_value)

    def test_nonexistent(self):
        return_value, stderr, _ = exec_pom_macro('%pom_xpath_disable nothing', {})
        assertIn(self, "Couldn't locate ", stderr)
        self.assertNotEqual(0, return_value)

    def test_disable_by_aid(self):
        return_value, stderr, report = exec_pom_macro(
               """%pom_xpath_disable "/pom:project/pom:artifactId='common'" """,
               poms_tree=example_tree,
               want_tree={'': 'parent_disabled_common.xml'})
        self.assertEqual(0, return_value, stderr)
        self.assertEqual('', report, report)

    def test_two_levels(self):
        return_value, stderr, report = exec_pom_macro(
                """%pom_xpath_disable "pom:parent[pom:artifactId='intermediate']" """,
               poms_tree=example_tree,
               want_tree={'intermediate': 'intermediate_disabled_schemas.xml'})
        self.assertEqual(0, return_value, stderr)
        self.assertEqual('', report, report)

    def test_skip_level(self):
        poms_tree = dict(example_tree)
        del poms_tree['intermediate/schemas']
        poms_tree['intermediate'] = 'intermediate_skip.xml'
        poms_tree['intermediate/src/schemas'] = 'example_schemas.xml'
        return_value, stderr, report = exec_pom_macro(
                """%pom_xpath_disable "pom:project.build.sourceEncoding='UTF-8'" """,
               poms_tree=poms_tree,
               want_tree={'intermediate': 'intermediate_disabled_schemas.xml'})
        self.assertEqual(0, return_value, stderr)
        self.assertEqual('', report, report)

    def test_fail_on_main(self):
        return_value, stderr, _ = exec_pom_macro(
                """%pom_xpath_disable "pom:groupId='com.example'" """,
               poms_tree=example_tree)
        assertIn(self, "Main POM satisfies the condition", stderr)
        self.assertNotEqual(0, return_value)

if __name__ == '__main__':
    unittest.main()



#
# Copyright (c) 2014, Red Hat, Inc.
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
#
# 1. Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in the
#    documentation and/or other materials provided with the
#    distribution.
# 3. Neither the name of Red Hat nor the names of its
#    contributors may be used to endorse or promote products derived
#    from this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
# A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
# OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
# LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
# DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
# THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#
# Authors:  Stanislav Ochotnicky <sochotnicky@redhat.com>

import optparse
import sys

from javapackages.maven.artifact import (Artifact, ArtifactFormatException,
                                         ArtifactValidationException)
from javapackages.xmvn.xmvn_config import XMvnConfig
from javapackages.common.util import args_to_unicode
from javapackages.common.exception import JavaPackagesToolsException


class SaneParser(optparse.OptionParser):
    def format_epilog(self, formatter):
        return self.epilog

usage = "usage: %prog [options] <MVN spec> <MVN spec> [MVN spec ...]"
epilog = """
MVN spec:
Specification of Maven artifact in following format:

      groupId:artifactId[:extension[:classifier]][:version]

Wildcards (*) and empty parts in specifications are allowed (treated as wildcard).

Examples of valid specifications:
commons-lang:commons-lang:1.2
commons-lang:commons-lang:war:
commons-lang:commons-lang:war:test-jar:
commons-lang:commons-lang:war:test-jar:3.1
*:commons-lang (equivalent to ':commons-lang')
"""

if __name__ == "__main__":
    parser = SaneParser(usage=usage,
                        epilog=epilog)

    sys.argv = args_to_unicode(sys.argv)

    (options, args) = parser.parse_args()
    if len(args) < 2:
        parser.error("At least 2 arguments are required")

    try:
        orig = Artifact.from_mvn_str(args[0])
        aliases = []
        for alias in args[1:]:
            aliases.append(Artifact.from_mvn_str(alias))

        XMvnConfig().add_aliases(orig, aliases)
    except (ArtifactValidationException, ArtifactFormatException) as e:
        parser.error("{e}: Provided artifact strings were invalid. "
                     "Please see help  and check your arguments".format(e=e))
        sys.exit(1)
    except JavaPackagesToolsException as e:
        sys.exit(e)



#
# Copyright (c) 2014-2016, Red Hat, Inc.
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
#
# 1. Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in the
#    documentation and/or other materials provided with the
#    distribution.
# 3. Neither the name of Red Hat nor the names of its
#    contributors may be used to endorse or promote products derived
#    from this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
# A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
# OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
# LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
# DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
# THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#
# Authors:  Michal Srb <msrb@redhat.com>

from __future__ import print_function

from javapackages.metadata.metadata import Metadata
from javapackages.metadata.artifact import MetadataArtifact
from javapackages.metadata.dependency import MetadataDependency

from javapackages.maven.artifact import Artifact, ArtifactFormatException
from javapackages.maven.pom import POM, PomLoadingException
from javapackages.ivy.ivyfile import IvyFile

from javapackages.xmvn.xmvn_resolve import (XMvnResolve, ResolutionRequest,
                                            XMvnResolveException)
from javapackages.common.util import args_to_unicode
from javapackages.common.exception import JavaPackagesToolsException

import sys
import os
import lxml.etree
from optparse import OptionParser


usage = "usage: %prog [options] <MVN spec | POM path> [artifact path]"
epilog = """
MVN spec:
Specification of Maven artifact in following format:

    groupId:artifactId[:extension[:classifier]][:version]

Wildcards (*) and empty parts in specifications are allowed (treated
as wildcard). JAR path must also be specified if this option is used.

Examples of valid specifications:
commons-lang:commons-lang:1.2
commons-lang:commons-lang:war:
commons-lang:commons-lang:war:test-jar:
commons-lang:commons-lang:war:test-jar:3.1
*:commons-lang (equivalent to ':commons-lang')

POM path:
Path where POM file is located.

Artifact path:
Path where Artifact file (usually JAR) is located.
"""


config = ".xmvn-reactor"


class ExtensionsDontMatch(JavaPackagesToolsException):
    pass


def get_parent_pom(pom):
    try:
        metadata = Metadata.create_from_file(config)
        known_artifacts = metadata.get_provided_artifacts()
        # TODO: implement __hash__() and __cmp__() in MetadataArtifact
        for artifact in known_artifacts:
            if (artifact.extension == "pom" and
               artifact.groupId == pom.groupId and
               artifact.artifactId == pom.artifactId):
                return POM(artifact.path)
    except IOError:
        pass

    req = ResolutionRequest(pom.groupId, pom.artifactId,
                            extension="pom", version=pom.version)
    result = XMvnResolve.process_raw_request([req])[0]
    if not result:
        raise XMvnResolveException("Unable to resolve parent POM {g}:{a}:{e}:{v}"
                                   .format(g=pom.groupId, a=pom.artifactId,
                                           e="pom", v=pom.version))

    return POM(result.artifactPath)


def is_it_ivy_file(fpath):
    """Try to determine whether file in given path is Ivy file or not"""
    et = lxml.etree.ElementTree()
    doc = et.parse(fpath)

    return doc.tag == "ivy-module"


def add_artifact_elements(metadata, art_template, ppath=None, jpath=None):
    for path in [ppath, jpath]:
        if path:
            art = art_template.copy()
            if path is ppath:
                if not is_it_ivy_file(ppath):
                    art.extension = "pom"
                else:
                    art.extension = "xml"
                    art.properties["type"] = "ivy"

            art.path = os.path.abspath(path)
            metadata.artifacts.append(art)


def merge_sections(main, update):
    for upd in update:
        for curr in main:
            if curr.compare_to(upd):
                curr.merge_with(upd)
                break
        else:
            main.append(upd)


def get_model_variables(pom):
    props = {}
    if pom.groupId:
        props["project.groupId"] = pom.groupId
    if pom.artifactId:
        props["project.artifactId"] = pom.artifactId
    if pom.version:
        props["project.version"] = pom.version
    return props


def expand_props(deps, props):
    for d in deps:
        d.interpolate(props)


def gather_dependencies(pom_path):
    if not pom_path:
        return []
    pom = POM(pom_path)
    pom_props = get_model_variables(pom)
    deps, depm, props = _get_dependencies(pom)
    # expand project model variables
    expand_props(deps, pom_props)
    expand_props(depm, pom_props)

    curr_pom = pom
    parent = pom.parent
    while parent:
        ppom = None
        if parent.relativePath:
            try:
                ppom_path = os.path.join(os.path.dirname(curr_pom._path),
                                         parent.relativePath)
                if os.path.isdir(ppom_path):
                    ppom_path = os.path.join(ppom_path, 'pom.xml')
                ppom = POM(ppom_path)
            except PomLoadingException:
                pass
        if not ppom:
            ppom = get_parent_pom(parent)

        parent = ppom.parent
        pom_props = get_model_variables(ppom)
        pdeps, pdepm, pprops = _get_dependencies(ppom)
        expand_props(pdeps, pom_props)
        expand_props(pdepm, pom_props)

        # merge "dependencies" sections
        merge_sections(deps, pdeps)
        # merge "dependencyManagement" sections
        merge_sections(depm, pdepm)

        # merge "properties" sections
        for pkey in pprops:
            if pkey not in props:
                props[pkey] = pprops[pkey]

        curr_pom = ppom

    for d in deps:
        d.interpolate(props)

    for dm in depm:
        dm.interpolate(props)

    # apply dependencyManagement on deps
    for d in deps:
        for dm in depm:
            if d.compare_to(dm):
                d.merge_with(dm)
                break

    # only deps with scope "compile" or "runtime" are interesting
    deps = [x for x in deps if x.scope in ["", "compile", "runtime"]]

    return deps


def _get_dependencies(pom):
    deps = []
    depm = []
    props = {}

    deps.extend([x for x in pom.dependencies])
    depm.extend([x for x in pom.dependencyManagement])
    props = pom.properties

    return deps, depm, props


def _main():
    OptionParser.format_epilog = lambda self, formatter: self.epilog
    parser = OptionParser(usage=usage,
                        epilog=epilog)
    parser.add_option("--skip-dependencies", action="store_true", default=False,
                      help="skip dependencies section in resulting metadata")
    parser.add_option("-D", action="append", type="str",
                      help="add artifact property", metavar="property=value")

    sys.argv = args_to_unicode(sys.argv)

    (options, args) = parser.parse_args()
    if len(args) < 1:
        parser.error("At least 1 argument is required")

    pom_path = None
    if os.path.exists(args[0]):
        pom_path = args[0]
        if is_it_ivy_file(pom_path):
            uart = IvyFile(pom_path)
        else:
            # it should be good old POM file
            uart = POM(pom_path)
    elif ':' in args[0]:
        try:
            uart = Artifact.from_mvn_str(args[0])
            uart.validate(allow_backref=False)
            if len(args) == 1:
                parser.error("When using artifact specification artifact path must be "
                             "provided")
            if not (uart.groupId and uart.artifactId and uart.version):
                parser.error("Defined artifact has to include at least groupId, "
                             "artifactId and version")
        except ArtifactFormatException as e:
            parser.error("Failed to parse artifact string: {0}".format(e))
    else:
        message = ("The first argument '{0}' doesn't point to an existing file "
                   "nor it looks like an artifact string").format(args[0])
        parser.error(message)


    art = MetadataArtifact(uart.groupId, uart.artifactId, version=uart.version)
    if hasattr(uart, "extension") and uart.extension:
        art.extension = uart.extension
    if hasattr(uart, "classifier") and uart.classifier:
        art.classifier= uart.classifier

    jar_path = None
    if len(args) > 1:
        jar_path = args[1]
        extension = (os.path.splitext(jar_path)[1])[1:]
        if hasattr(art, "extension") and art.extension and art.extension != extension and not pom_path:
            raise ExtensionsDontMatch("Extensions don't match: '%s' != '%s'" % (art.extension, extension))
        else:
            art.extension = extension
    else:
        art.extension = "pom"

    if os.path.exists(config):
        metadata = Metadata.create_from_file(config)
    else:
        metadata = Metadata()

    if (not options.skip_dependencies and pom_path
       and not is_it_ivy_file(pom_path)):
        deps = []
        mvn_deps = gather_dependencies(pom_path)
        for d in mvn_deps:
            deps.append(MetadataDependency.from_mvn_dependency(d))
        if deps:
            art.dependencies = set(deps)
    else:
        art.properties["xmvn.resolver.disableEffectivePom"] = "true"

    if options.D:
        for d_opt in options.D:
            key, value = d_opt.split("=")
            art.properties[key] = value

    add_artifact_elements(metadata, art, pom_path, jar_path)

    metadata.write_to_file(config)

if __name__ == "__main__":
    try:
        _main()
    except JavaPackagesToolsException as e:
        sys.exit(e)



#
# Copyright (c) 2014, Red Hat, Inc.
#
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
#
# 1. Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in the
#    documentation and/or other materials provided with the
#    distribution.
# 3. Neither the name of the Red Hat nor the names of its
#    contributors may be used to endorse or promote products derived
#    from this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
# A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
# OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
# LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
# DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
# THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#
# Authors:  Michal Srb <msrb@redhat.com>

import javapackages.maven.pomreader as POMReader
from javapackages.maven.pomreader import PomLoadingException

import os


class IvyFile(object):
    """
    Class for querying basic information from ivy.xml files used by Apache Ivy
    """
    def __init__(self, path):
        self.__doc = POMReader.load(path)
        self._path = os.path.join(path)

    def __str__(self):
        return ":".join([self.groupId, self.artifactId, self.version])

    @property
    def parent(self):
        return None

    @property
    def parentGroupId(self):
        return None

    @property
    def parentArtifactId(self):
        return None

    @property
    def parentVersion(self):
        return None

    @property
    def groupId(self):
        gId = POMReader.find(self.__doc, '/ivy-module/info')
        if gId is not None:
            try:
                gId = gId.attrib["organisation"]
            except:
                raise PomLoadingException("Unable to determine groupId")
        return gId

    @property
    def artifactId(self):
        aId = POMReader.find(self.__doc, '/ivy-module/info')
        if aId is not None:
            try:
                aId = aId.attrib["module"]
            except KeyError:
                raise PomLoadingException("Unable to determine artifactId")
        return aId

    @property
    def version(self):
        version = POMReader.find(self.__doc, '/ivy-module/info')
        if version is not None:
            try:
                version = version.attrib["revision"]
            except KeyError:
                raise PomLoadingException("Unable to determine version")
        return version

    # only for compatibility with POM, this will be removed in future
    @property
    def packaging(self):
        return None

    @property
    def dependencies(self):
        return []

    @property
    def dependencyManagement(self):
        return []

    @property
    def pluginManagement(self):
        return []

    @property
    def plugins(self):
        return []

    @property
    def extensions(self):
        return []

    @property
    def properties(self):
        return {}



#
# Copyright (c) 2014, Red Hat, Inc.
#
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
#
# 1. Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in the
#    documentation and/or other materials provided with the
#    distribution.
# 3. Neither the name of the Red Hat nor the names of its
#    contributors may be used to endorse or promote products derived
#    from this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
# A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
# OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
# LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
# DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
# THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#
# Authors:  Michal Srb <msrb@redhat.com>

from javapackages.maven.artifact import AbstractArtifact, ArtifactFormatException
import javapackages.maven.pomreader as POMReader


class Exclusion(AbstractArtifact):

    def __init__(self, groupId, artifactId):
        self.groupId = groupId.strip()
        self.artifactId = artifactId.strip()

    @classmethod
    def from_xml_element(cls, xmlnode):
        """
        Create Exclusion from xml.etree.ElementTree.Element as contained
        within pom.xml.
        """

        parts = {'groupId': '', 'artifactId': ''}
        parts = POMReader.find_parts(xmlnode, parts)

        if not parts['groupId'] or not parts['artifactId']:
            raise ArtifactFormatException(
                "Empty groupId or artifactId encountered. "
                "This is a bug, please report it!")

        return cls(parts['groupId'], parts['artifactId'])

    @classmethod
    def from_mvn_str(cls, mvnstr):
        """
        Create Exclusion from Maven-style definition

        The string should be in the format of:
           groupId:artifactId
        """
        p = cls.get_parts_from_mvn_str(mvnstr)
        return cls(p['groupId'], p['artifactId'])



#
# Copyright (c) 2015, Red Hat, Inc.
#
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
#
# 1. Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in the
#    documentation and/or other materials provided with the
#    distribution.
# 3. Neither the name of the Red Hat nor the names of its
#    contributors may be used to endorse or promote products derived
#    from this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
# A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
# OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
# LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
# DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
# THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#
# Authors:  Michal Srb <msrb@redhat.com>

from javapackages.maven.artifact import (AbstractArtifact,
                                         ArtifactFormatException)
from javapackages.maven.exclusion import Exclusion
import javapackages.maven.pomreader as POMReader
from lxml.etree import Element, SubElement


class Dependency(AbstractArtifact):
    """Maven dependency."""

    def __init__(self, groupId, artifactId, extension="", classifier="",
                 version="", scope="", optional="", exclusions=None):
        # groupId and artifactId are mandatory
        if groupId is not None:
            self.groupId = groupId.strip()
        if artifactId is not None:
            self.artifactId = artifactId.strip()

        if not self.groupId or not self.artifactId:
            raise ArtifactFormatException("\"{g}:{a}\" is not a valid artifact"
                                          .format(g=self.groupId,
                                                  a=self.artifactId))

        # default values
        self.extension = "jar"
        self.classifier = ""
        self.version = ""
        self.scope = "compile"
        self.optional = "false"
        self.exclusions = []

        self._default_scope = True
        self._default_optional = True

        if extension:
            self.extension = extension.strip()
        if classifier:
            self.classifier = classifier.strip()
        if version:
            self.version = version.strip()
        if scope is not None:
            self.scope = scope.strip()
            self._default_scope = False
        if optional is not None:
            self.optional = optional.strip()
            self._default_optional = False
        if exclusions:
            self.exclusions = exclusions

    def is_optional(self):
        """Return True if the dependency is optional, False otherwise."""
        if self.optional and self.optional.lower() == "true":
            return True
        return False

    def get_xml_element(self, root="dependency"):
        """Return XML Element node representation of this dependency."""
        xml_root = AbstractArtifact.get_xml_element(self, root)

        if not self._default_scope:
            item = SubElement(xml_root, "scope")
            item.text = self.scope

        if not self._default_optional:
            item = SubElement(xml_root, "optional")
            item.text = self.optional

        if self.exclusions:
            exc_root = Element("exclusions")
            for exc in self.exclusions:
                exc_root.insert(len(exc_root), exc.get_xml_element())
            xml_root.insert(len(root), exc_root)

        return xml_root

    @classmethod
    def from_xml_element(cls, xmlnode):
        """
        Create Dependency from xml.etree.ElementTree.Element as contained
        within pom.xml.
        """

        parts = {"groupId": "",
                 "artifactId": "",
                 "type": "",
                 "classifier": "",
                 "version": "",
                 "scope": "",
                 "optional": ""}

        parts = POMReader.find_raw_parts(xmlnode, parts)

        # exclusions
        excnodes = xmlnode.findall("{*}exclusions/{*}exclusion")

        exclusions = []
        for exc in [Exclusion.from_xml_element(x) for x in excnodes]:
            exclusions.append(exc)

        return cls(parts["groupId"],
                   parts["artifactId"],
                   extension=parts["type"],
                   classifier=parts["classifier"],
                   version=parts["version"],
                   scope=parts["scope"],
                   optional=parts["optional"],
                   exclusions=exclusions)

    @classmethod
    def from_mvn_str(cls, mvnstr):
        """
        Create Dependency from Maven-style definition.

        The string should be in the format of:
           groupId:artifactId[:extension[:classifier]][:version]

        Where last part is always considered to be version unless empty.
        """
        parts = cls.get_parts_from_mvn_str(mvnstr)
        return cls(parts['groupId'],
                   parts['artifactId'],
                   extension=parts['extension'],
                   classifier=parts['classifier'],
                   version=parts['version'])



import unittest
import shutil
import os
from test_common import javautils_script, get_config_file_list, \
        get_actual_config, get_expected_config, DIRPATH

from xml_compare import compare_xml_files

class TestMvnConfig(unittest.TestCase):

    maxDiff = 2048

    def setUp(self):
        self.olddir = os.getcwd()
        self.workdir = os.path.join(DIRPATH, 'workdir')
        os.mkdir(self.workdir)
        os.chdir(self.workdir)

    def tearDown(self):
        try:
            shutil.rmtree(self.workdir)
        except OSError:
            pass
        os.chdir(self.olddir)

    @javautils_script('mvn_config', [])
    def test_run_no_args(self, stdout, stderr, return_value):
        self.assertNotEqual(return_value, 0)
        self.assertEqual("Usage:", stderr[:6])

    @javautils_script('mvn_config', ['-h'])
    def test_help(self, stdout, stderr, return_value):
        self.assertTrue(stdout)

    @javautils_script('mvn_config',['aaa', ])
    def test_single(self, stdout, stderr, return_value):
        self.assertNotEqual(return_value, 0)
        self.assertTrue(stderr)

    @javautils_script('mvn_config',['a', 'b', 'c', ])
    def test_more(self, stdout, stderr, return_value):
        self.assertNotEqual(return_value, 0)
        self.assertTrue(stderr)

    @javautils_script('mvn_config',['aaa', 'bbb', ])
    def test_simple(self, stdout, stderr, return_value):
        self.assertEqual(return_value, 0, stderr)
        filelist = get_config_file_list()
        self.assertEqual(len(filelist), 1)
        for filename in filelist:
            report = compare_xml_files(get_actual_config(filename),
                 get_expected_config(filename, 'mvn_config', 'simple'),
                 ['artifactGlob'])
            self.assertFalse(report, '\n' + report)

    @javautils_script('mvn_config',['a/b/c', 'xxx', ])
    def test_path(self, stdout, stderr, return_value):
        self.assertEqual(return_value, 0, stderr)
        filelist = get_config_file_list()
        self.assertEqual(len(filelist), 1)
        for filename in filelist:
            report = compare_xml_files(get_actual_config(filename),
                 get_expected_config(filename, 'mvn_config', 'path'),
                 ['artifactGlob'])
            self.assertFalse(report, '\n' + report)

    @javautils_script('mvn_config',['a', '<b/>', ])
    def test_xml1(self, stdout, stderr, return_value):
        self.assertEqual(return_value, 0, stderr)
        filelist = get_config_file_list()
        self.assertEqual(len(filelist), 1)
        for filename in filelist:
            report = compare_xml_files(get_actual_config(filename),
                 get_expected_config(filename, 'mvn_config', 'xml1'),
                 ['artifactGlob'])
            self.assertFalse(report, '\n' + report)

    @javautils_script('mvn_config',['a', '<b>c</b>', ])
    def test_xml2(self, stdout, stderr, return_value):
        self.assertEqual(return_value, 0, stderr)
        filelist = get_config_file_list()
        self.assertEqual(len(filelist), 1)
        for filename in filelist:
            report = compare_xml_files(get_actual_config(filename),
                 get_expected_config(filename, 'mvn_config', 'xml2'),
                 ['artifactGlob'])
            self.assertFalse(report, '\n' + report)

    @javautils_script('mvn_config',['a', '<b>c</b><d/>', ])
    def test_xml3(self, stdout, stderr, return_value):
        self.assertEqual(return_value, 0, stderr)
        filelist = get_config_file_list()
        self.assertEqual(len(filelist), 1)
        for filename in filelist:
            report = compare_xml_files(get_actual_config(filename),
                 get_expected_config(filename, 'mvn_config', 'xml3'),
                 ['artifactGlob'])
            self.assertFalse(report, '\n' + report)

    @javautils_script('mvn_config',['a', '<b>c</b><d>e</d>', ])
    def test_xml4(self, stdout, stderr, return_value):
        self.assertEqual(return_value, 0, stderr)
        filelist = get_config_file_list()
        self.assertEqual(len(filelist), 1)
        for filename in filelist:
            report = compare_xml_files(get_actual_config(filename),
                 get_expected_config(filename, 'mvn_config', 'xml4'),
                 ['artifactGlob'])
            self.assertFalse(report, '\n' + report)

    @javautils_script('mvn_config',['a', '<b><c>d</c></b>', ])
    def test_nested_xml1(self, stdout, stderr, return_value):
        self.assertEqual(return_value, 0, stderr)
        filelist = get_config_file_list()
        self.assertEqual(len(filelist), 1)
        for filename in filelist:
            report = compare_xml_files(get_actual_config(filename),
                 get_expected_config(filename, 'mvn_config', 'nested_xml1'),
                 ['artifactGlob'])
            self.assertFalse(report, '\n' + report)

    @javautils_script('mvn_config',['a', '<b><c>d</c>d</b>', ])
    def test_nested_xml2(self, stdout, stderr, return_value):
        self.assertEqual(return_value, 0, stderr)
        filelist = get_config_file_list()
        self.assertEqual(len(filelist), 1)
        for filename in filelist:
            report = compare_xml_files(get_actual_config(filename),
                 get_expected_config(filename, 'mvn_config', 'nested_xml2'),
                 ['artifactGlob'])
            self.assertFalse(report, '\n' + report)

    @javautils_script('mvn_config',['a', '<b', ])
    def test_invalid_xml1(self, stdout, stderr, return_value):
        self.assertNotEqual(return_value, 0)
        self.assertTrue(stderr)

    @javautils_script('mvn_config',['a', '<b>', ])
    def test_invalid_xml2(self, stdout, stderr, return_value):
        self.assertNotEqual(return_value, 0)
        self.assertTrue(stderr)

    @javautils_script('mvn_config',['a', '<b><c></b>', ])
    def test_invalid_xml3(self, stdout, stderr, return_value):
        self.assertNotEqual(return_value, 0)
        self.assertTrue(stderr)

    @javautils_script('mvn_config',['a', '<b></c></b>', ])
    def test_invalid_xml4(self, stdout, stderr, return_value):
        self.assertNotEqual(return_value, 0)
        self.assertTrue(stderr)

    @javautils_script('mvn_config',['a', '<b>c&lt;d</b>', ])
    def test_entity(self, stdout, stderr, return_value):
        self.assertEqual(return_value, 0, stderr)
        filelist = get_config_file_list()
        self.assertEqual(len(filelist), 1)
        for filename in filelist:
            report = compare_xml_files(get_actual_config(filename),
                 get_expected_config(filename, 'mvn_config', 'entity'),
                 ['artifactGlob'])
            self.assertFalse(report, '\n' + report)

    @javautils_script('mvn_config',['a', 'f<b>c</b>d', ])
    def test_mixed(self, stdout, stderr, return_value):
        self.assertEqual(return_value, 0, stderr)
        filelist = get_config_file_list()
        self.assertEqual(len(filelist), 1)
        for filename in filelist:
            report = compare_xml_files(get_actual_config(filename),
                 get_expected_config(filename, 'mvn_config', 'mixed'),
                 ['artifactGlob'])
            self.assertFalse(report, '\n' + report)

if __name__ == '__main__':
    unittest.main()



#
# Copyright (c) 2015, Red Hat, Inc.
#
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
#
# 1. Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in the
#    documentation and/or other materials provided with the
#    distribution.
# 3. Neither the name of the Red Hat nor the names of its
#    contributors may be used to endorse or promote products derived
#    from this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
# A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
# OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
# LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
# DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
# THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#
# Authors:  Michael Simacek <msimacek@redhat.com>

from __future__ import print_function

import sys

from javapackages.common.mock import (InstallationException, ConnectionException,
                                      install_maven_artifact)

if __name__ == '__main__':
    if len(sys.argv) != 3:
        print("Usage: artifact-type artifact-coordinates", file=sys.stderr)
        sys.exit(1)
    try:
        if sys.argv[1] == 'maven':
            install_maven_artifact(sys.argv[2])
        else:
            print("Unsupported artifact type", file=sys.stderr)
            sys.exit(1)
    except InstallationException as e:
        print("Cannot install {dep}".format(dep=e.dep), file=sys.stderr)
        print(e.out, file=sys.stderr)
        sys.exit(2)
    except ConnectionException:
        sys.exit(3)



#
# Copyright (c) 2014, Red Hat, Inc.
#
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
#
# 1. Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in the
#    documentation and/or other materials provided with the
#    distribution.
# 3. Neither the name of the Red Hat nor the names of its
#    contributors may be used to endorse or promote products derived
#    from this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
# A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
# OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
# LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
# DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
# THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#
# Authors:  Alexander Kurtakov <akurtako@redhat.com>
#           Michal Srb <msrb@redhat.com>

import os

import javapackages.common.config as config
from javapackages.common.osgi import OSGiBundle
from javapackages.cache.cache import Cache
from javapackages.cache.metadata import MetadataCache


class OSGiCache(Cache):

    def __init__(self, rpmconf):
        super(OSGiCache, self).__init__(rpmconf)
        self._config_name = config.osgi_cache_f
        self._cache = self._read_cache()
        self._metadata_cache = MetadataCache(rpmconf)

        if self._cache is None:
            self._cache = self._process_buildroot()
            self._write_cache(self._cache)

    def get_bundle_for_path(self, path):
        try:
            return self._cache[path]
        except KeyError:
            pass
        return None

    def get_bundle(self, name):
        for bundle in self._cache.values():
            if bundle.bundle == name:
                return bundle
        return None

    def _process_buildroot(self):
        # "path: OSGiBundle" mapping
        cache = {}

        bundle_paths = self._find_paths()
        for path in bundle_paths:
            artifact = self._metadata_cache.get_artifact_for_path(path, can_be_dir=True)
            if artifact and artifact.has_osgi_information():
                bundle = artifact.get_osgi_bundle()
            else:
                bundle = OSGiBundle.from_manifest(path)
                if bundle:
                    if not bundle.namespace and self._scl:
                        bundle.namespace = self._scl
            if bundle:
                cache.update({path: bundle})

        return cache

    def _check_path(self, path):
        if os.path.islink(path):
            return False
        if path.endswith(".jar"):
            return True
        if path.endswith("/MANIFEST.MF"):
            # who knows where the manifest can be in buildroot.
            # this is an attempt to identify only MANIFEST.MF files
            # which are in %{_datadir} or %{_prefix}/lib
            if "/usr/share/" in path or "/usr/lib" in path:
                return True
        return False

    def check_path_in_metadata(self, path):
        artifact = self._metadata_cache.get_artifact_for_path(path,
                                                              can_be_dir=True)
        if artifact and artifact.has_osgi_information():
            return True
        return False



import unittest

from test_common import javadocreq, assertIn


class TestJavadocReq(unittest.TestCase):

    @javadocreq(["something", "that's ignored anyway"])
    def test_javadoc(self, stdout, stderr, return_value):
        self.assertEqual(return_value, 0, stderr)
        self.assertEqual("javapackages-tools\n", stdout)

    @javadocreq([], javaconfdirs=["javadoc/first", "javadoc/second"])
    def test_configuration(self, stdout, stderr, return_value):
        self.assertEqual(return_value, 0, stderr)
        sout = [x for x in stdout.split('\n') if x]
        self.assertEqual(len(sout), 1)
        assertIn(self, "SCL-javapackages-tools", sout)

    @javadocreq([], javaconfdirs=["non-existent"])
    def test_missing_configuration(self, stdout, stderr, return_value):
        self.assertEqual(return_value, 0, stderr)
        self.assertEqual("", stdout)

if __name__ == '__main__':
    unittest.main()



#
# Copyright (c) 2015, Red Hat, Inc.
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
#
# 1. Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in the
#    documentation and/or other materials provided with the
#    distribution.
# 3. Neither the name of Red Hat nor the names of its
#    contributors may be used to endorse or promote products derived
#    from this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
# A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
# OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
# LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
# DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
# THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#
# Authors:  Michael Simacek <msimacek@redhat.com>

import six

from lxml import etree
from copy import deepcopy

from javapackages.common.exception import JavaPackagesToolsException

class XMLBindingException(JavaPackagesToolsException):
    pass


def _get_item_type(spec):
    assert 0 < len(spec) <= 2, spec
    spec = tuple(spec)
    ret = spec[0]
    if len(spec) == 1:
        if isinstance(spec[0], six.string_types):
            ret = str
    elif isinstance(spec[0], six.string_types):
        ret = spec[1]
    assert isinstance(ret, type), ret
    return ret


def _get_item_name(spec):
    assert 0 < len(spec) <= 2, spec
    spec = tuple(spec)
    ret = spec[0]
    if isinstance(spec[0], type) and issubclass(spec[0], ObjectBinding):
        ret = spec[0].element_name
    elif len(spec) == 2 and isinstance(spec[0], type):
        ret = spec[1]
    assert isinstance(ret, six.string_types), ret
    return ret


def _is_element(node):
    return isinstance(node.tag, six.string_types)

def _localname(element):
    return etree.QName(element.tag).localname

def from_element(for_type, element):
    if for_type is str:
        return element.text.strip() if element.text is not None else None
    if isinstance(for_type, list):
        item_type = _get_item_type(for_type)
        return [from_element(item_type, child) for child in element
                if _is_element(child)]
    if isinstance(for_type, set):
        item_type = _get_item_type(for_type)
        return set([from_element(item_type, child) for child in element
                    if _is_element(child)])
    if for_type is dict:
        new = {}
        for child in element:
            if isinstance(child.tag, six.string_types):
                name = _localname(child)
                value = from_element(str, child)
                new[name] = value
        return new
    if isinstance(for_type, type) and issubclass(for_type, ObjectBinding):
        name = _localname(element)
        if name != for_type.element_name:
            raise XMLBindingException("Unexpected element " + name)
        new = {}
        for child in element:
            if _is_element(child):
                name = _localname(child)
                if name in for_type.fields:
                    value = from_element(for_type.types.get(name, str), child)
                    if name in new:
                        raise XMLBindingException("More values for element " + name)
                    new[name] = value
        return for_type(**new)
    raise XMLBindingException("Unrecognized binding type: {0}".format(for_type))


def _make_element(name, ns=None):
    if ns:
        name = '{' + ns + '}' + name
    return etree.Element(name)

def to_element(obj, name=None, type_spec=None, ns=None):
    if isinstance(obj, six.string_types):
        element = _make_element(name, ns=ns)
        element.text = obj
        return element
    if isinstance(obj, list) or isinstance(obj, set):
        element = _make_element(name, ns=ns)
        for item in obj:
            element.append(to_element(item, name=_get_item_name(type_spec),
                                      ns=ns))
        return element
    if isinstance(obj, dict):
        element = _make_element(name, ns=ns)
        for key, value in sorted(obj.items()):
            # TODO rly?
            # entry = _make_element(key, ns=ns)
            entry = _make_element(key)
            entry.text = value
            element.append(entry)
        return element
    if isinstance(obj, ObjectBinding):
        name = obj.element_name
        ns = obj.xmlns or ns
        element = _make_element(name, ns=ns)
        for item_name in obj.fields:
            value = obj.values.get(item_name)
            if value:
                child = to_element(value, item_name,
                                   obj.types.get(item_name, str), ns=ns)
                element.append(child)
        return element

class ObjectBinding(object):
    element_name = None
    fields = []
    types = {}
    equality = None
    defaults = {}
    xmlns = None

    def __init__(self, *args, **kwargs):
        assert self.element_name
        assert self.fields
        self.values = self.defaults.copy()
        self._touched = set()
        for name in self.fields:
            item_type = self.types.get(name, str)
            if name not in self.values:
                if type(item_type) in (list, set):
                    self.values[name] = type(item_type)()
                elif item_type == dict:
                    self.values[name] = {}
                elif item_type == str:
                    self.values[name] = ''
        values = list(zip(self.fields, args)) + list(kwargs.items())
        for name, value in values:
            setattr(self, name, value)

    def __getattr__(self, name):
        if name in self.fields:
            return self.values.get(name)
        return getattr(super(ObjectBinding, self), name)

    def __setattr__(self, name, value):
        if name in self.fields:
            self.values[name] = value
            self._touched.add(name)
        else:
            super(ObjectBinding, self).__setattr__(name, value)

    def __contains__(self, name):
        return name in self._touched

    def __repr__(self):
        return repr(self.values)

    def _get_values_for_equality(self):
        if self.equality is not None:
            return [v for k, v in self.values.items() if k in self.equality]
        return [v for k, v in self.values.items()]

    def __eq__(self, that):
        return self is that or (type(self) is type(that) and
                                self._get_values_for_equality()
                                == that._get_values_for_equality())

    def __ne__(self, that):
        return not self.__eq__(that)

    def __hash__(self):
        if self.equality:
            return hash(self._get_values_for_equality)
        return id(self)

    def copy(self):
        return deepcopy(self)



#
# Copyright (c) 2015, Red Hat, Inc.
#
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
#
# 1. Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in the
#    documentation and/or other materials provided with the
#    distribution.
# 3. Neither the name of the Red Hat nor the names of its
#    contributors may be used to endorse or promote products derived
#    from this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
# A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
# OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
# LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
# DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
# THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#
# Authors:  Michal Srb <msrb@redhat.com>

import os
import logging
import pickle
import javapackages.common.config as config


class Cache(object):
    def __init__(self, rpmconf):
        self._cachedir = rpmconf.cachedir
        self._rpm_pid = rpmconf.rpm_pid
        self._scl = rpmconf.scl
        self._fresh = False
        self._config_name = "default_cache"

    def _process_buildroot(self):
        cache = {}
        # TODO: implement in subclass
        return cache

    def _find_paths(self):
        buildroot = config.get_buildroot()
        paths = []
        for dirpath, _, filenames in os.walk(buildroot):
            for filename in filenames:
                fpath = os.path.abspath(os.path.join(dirpath, filename))
                if self._check_path(fpath):
                    paths.append(fpath)
        return paths

    def _check_path(self, path):
        # TODO: implement in subclass
        return False

    def _read_cache(self):
        try:
            cachepath = os.path.join(self._cachedir, self._config_name)
            cachefile = open(cachepath, 'rb')
            cached_pid, cache = pickle.load(cachefile)
            cachefile.close()
            # check if the cache was most likely created during current build
            if cached_pid != self._rpm_pid:
                logging.warning("Cache in {path} is outdated, skipping"
                                .format(path=cachepath))
                return None
        except IOError:
            return None
        return cache

    def _write_cache(self, cache):
        try:
            cachefile = open(os.path.join(self._cachedir,
                                          self._config_name), 'wb')
            content = (self._rpm_pid, cache)
            pickle.dump(content, cachefile)
            cachefile.close()
            self._fresh = True
        except IOError:
            return None
        return cache

    def is_fresh(self):
        """
        Returns True if the cache was created during creation of this object,
        False otherwise.
        """
        return self._fresh



#
# Copyright (c) 2014, Red Hat, Inc.
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
#
# 1. Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in the
#    documentation and/or other materials provided with the
#    distribution.
# 3. Neither the name of Red Hat nor the names of its
#    contributors may be used to endorse or promote products derived
#    from this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
# A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
# OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
# LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
# DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
# THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#
# Authors:  Michal Srb <msrb@redhat.com>

import os
import lxml.etree

from javapackages.common.util import execute_command, command_exists
from javapackages.common.exception import JavaPackagesToolsException


class XMvnResolveException(JavaPackagesToolsException):
    pass


class XMvnResolve(object):
    # TODO:
    # - documentation
    tool = "xmvn-resolve"

    @staticmethod
    def is_available():
        # always return True if running in test environment
        if os.environ.get("JAVAPACKAGES_XMVN_RESOLVE_TEST", None) is not None:
            return True
        return command_exists(XMvnResolve.tool)

    @staticmethod
    def process_raw_request(raw_request_list):
        if not raw_request_list:
            return []
        command = "{tool} {args}".format(tool=XMvnResolve.tool,
                                         args="--raw-request")
        request = XMvnResolve._join_raw_requests(raw_request_list)
        test_env = os.environ.get("JAVAPACKAGES_XMVN_RESOLVE_TEST", None)
        if not test_env:
            rc, stdout, stderr = execute_command(command, input=request)
            if rc:
                raise XMvnResolveException("xmvn-resolve failed:\n" + stderr)
        else:
            stdout = ""
            with open(test_env, "rb") as f:
                lines = f.readlines()
                stdout = lines[0].decode("UTF-8")
                lines = lines[1:]
            with open(test_env, "wb") as f:
                f.write(b"\n".join(lines))

        result = XMvnResolve._process_results(stdout)
        return result

    @staticmethod
    def _join_raw_requests(raw_request_list):
        request = "<requests>"
        for r in raw_request_list:
            request += r.get_xml()
        request += "</requests>"

        return request

    @staticmethod
    def _process_results(result_xml):
        results = []

        doc = lxml.etree.fromstring(result_xml.encode("UTF-8"))
        nodes = doc.xpath('/results/result')
        for node in nodes:
            if len(node) > 0:
                ns = node.find('./namespace')
                if ns is not None:
                    ns = ns.text
                compat_ver = node.find('./compatVersion')
                if compat_ver is not None:
                    compat_ver = compat_ver.text
                path = node.find('./artifactPath')
                if path is not None:
                    path = path.text
                res = ResolutionResult(namespace=ns or "",
                                       compatVersion=compat_ver or "",
                                       path=path or "")
                results.append(res)
            else:
                results.append(None)
        return results


class ResolutionResult(object):
    def __init__(self, namespace="", compatVersion="", path=""):
        self.namespace = namespace
        self.compatVersion = compatVersion
        self.artifactPath = path

    def __str__(self):
        return "version:" + self.compatVersion + "namespace: " + self.namespace


class ResolutionRequest(object):
    def __init__(self, groupId, artifactId, extension="",
                 classifier="", version=""):
        self.groupId = groupId
        self.artifactId = artifactId
        self.extension = extension
        self.classifier = classifier
        self.version = version

    def get_xml(self):
        return ResolutionRequest.create_raw_request_xml(self.groupId,
                                                        self.artifactId,
                                                        self.extension,
                                                        self.classifier,
                                                        self.version)

    @staticmethod
    def create_raw_request_xml(groupId, artifactId, extension="",
                               classifier="", version=""):
        template = """
<request>
    <artifact>
        <groupId>{gid}</groupId>
        <artifactId>{aid}</artifactId>{ext}{cla}{ver}
    </artifact>
</request>
"""
        ver = ""
        cla = ""
        ext = ""
        if extension:
            ext = "<extension>{ext}</extension>".format(ext=extension)
        if classifier:
            cla = "<classifier>{cla}</classifier>".format(cla=classifier)
        if version:
            ver = "<version>{ver}</version>".format(ver=version)

        return template.format(gid=groupId, aid=artifactId,
                               ext=ext, cla=cla, ver=ver)

    @classmethod
    def from_artifact(cls, artifact):
        return cls(artifact.groupId, artifact.artifactId,
                   artifact.extension, artifact.classifier, artifact.version)



import unittest

from javapackages.common.osgi import OSGiRequire, OSGiBundle


class TestOSGi(unittest.TestCase):

    def test_bundle_simple(self):
        b = OSGiBundle.from_string("bundle.name 1")
        self.assertEqual(b.bundle, "bundle.name")
        self.assertEqual(b.version, "1")
        self.assertEqual(b.namespace, "")
        self.assertEqual(b.get_rpm_str(), "osgi(bundle.name) = 1")

    def test_bundle_namespace(self):
        b = OSGiBundle.from_string("bundle.name(ns) 1")
        self.assertEqual(b.bundle, "bundle.name")
        self.assertEqual(b.version, "1")
        self.assertEqual(b.namespace, "ns")
        self.assertEqual(b.get_rpm_str(), "ns-osgi(bundle.name) = 1")

    def test_bundle_requires(self):
        b = OSGiBundle.from_string("bundle.name() 1 req1.abc,req2(ns)")
        self.assertEqual(b.bundle, "bundle.name")
        self.assertEqual(b.version, "1")
        self.assertEqual(b.namespace, "")
        self.assertEqual(b.get_rpm_str(), "osgi(bundle.name) = 1")
        self.assertEqual(len(b.requires), 2)

    def test_bundle_whitespaces(self):
        b = OSGiBundle.from_string("  bundle.name()   1 req1.abc,req2(ns) ")
        self.assertEqual(b.bundle, "bundle.name")
        self.assertEqual(b.version, "1")
        self.assertEqual(b.namespace, "")
        self.assertEqual(b.get_rpm_str(), "osgi(bundle.name) = 1")
        self.assertEqual(len(b.requires), 2)

    def test_bundle_from_properties(self):
        props = {"osgi.id": "bundle.name", "osgi.version": "1"}
        b = OSGiBundle.from_properties(props)
        self.assertEqual(b.bundle, "bundle.name")
        self.assertEqual(b.version, "1")
        self.assertEqual(b.namespace, "")
        self.assertEqual(b.get_rpm_str(), "osgi(bundle.name) = 1")
        self.assertEqual(len(b.requires), 0)

    def test_bundle_from_properties_complex(self):
        props = {"osgi.id": "bundle.name",
                 "osgi.version": "1",
                 "osgi.requires": "req1.abc,req2(ns2)",
                 "osgi.namespace": "ns1"
                 }
        b = OSGiBundle.from_properties(props)
        self.assertEqual(b.bundle, "bundle.name")
        self.assertEqual(b.version, "1")
        self.assertEqual(b.namespace, "ns1")
        self.assertEqual(b.get_rpm_str(), "ns1-osgi(bundle.name) = 1")
        self.assertEqual(len(b.requires), 2)

        self.assertEqual(b.requires[0].bundle, "req1.abc")
        self.assertEqual(b.requires[0].namespace, "")
        self.assertEqual(b.requires[0].get_rpm_str(), "osgi(req1.abc)")

        self.assertEqual(b.requires[1].bundle, "req2")
        self.assertEqual(b.requires[1].namespace, "ns2")
        self.assertEqual(b.requires[1].get_rpm_str(), "ns2-osgi(req2)")

    def test_require(self):
        r = OSGiRequire.from_string("osgi.req")
        self.assertEqual(r.bundle, "osgi.req")
        self.assertEqual(r.namespace, "")
        self.assertEqual(r.get_rpm_str(), "osgi(osgi.req)")
        self.assertEqual(r.get_rpm_str("1"), "osgi(osgi.req) = 1")

    def test_require_namespace(self):
        r = OSGiRequire.from_string("osgi.req(ns)")
        self.assertEqual(r.bundle, "osgi.req")
        self.assertEqual(r.namespace, "ns")
        self.assertEqual(r.get_rpm_str(), "ns-osgi(osgi.req)")
        self.assertEqual(r.get_rpm_str("1"), "ns-osgi(osgi.req) = 1")


if __name__ == '__main__':
    unittest.main()



import inspect
from zipfile import ZipFile
import six
import os
import unittest
import shutil
from test_common import (DIRPATH, mvn_depmap, call_script,
                         prepare_metadata, assertIn)
from test_rpmbuild import Package

from lxml import etree
from xml_compare import compare_lxml_etree


class TestMavenDepmap(unittest.TestCase):

    maxDiff = 2048

    def setUp(self):
        try:
            self.olddir = os.getcwd()
            self.datadir = os.path.join(DIRPATH,
                                        'data',
                                        'maven_depmap')
            self.workdir = os.path.realpath(os.path.join(self.datadir, "..",
                                            "maven_depmap_workdir"))

            shutil.copytree(self.datadir, self.workdir)
            os.chdir(self.workdir)
        except OSError:
            pass

        prepare_metadata(self.workdir)

    def tearDown(self):
        try:
            shutil.rmtree(self.workdir)
            os.chdir(self.olddir)
        except OSError:
            pass

    def check_result(self, test_name, depmap):
        got = etree.fromstring(depmap)
        want = etree.parse(os.path.join(self.workdir,
                                        test_name+"-want.xml")).getroot()
        report = compare_lxml_etree(got, want, unordered=['artifact', 'maven'])
        if report:
            report = '\n' + report
        return report

    def check_archive(self, test_name, archive_path, keep_comments=False):
        got = ZipFile(archive_path, 'r')
        want = ZipFile('{name}-want.{ext}'.format(name=test_name,
                       ext=archive_path.split('.')[-1]))
        try:
            if got.testzip() is not None:
                return ("Not valid zip file", "")
            got_mf = self.read_archive(got, keep_comments)
            want_mf = self.read_archive(want, keep_comments)
        finally:
            got.close()
            want.close()
        return (got_mf, want_mf)

    def read_archive(self, archive, keep_comments=False):
        res = {}
        for filename in archive.namelist():
            mf_file = archive.open(filename)
            try:
                if (keep_comments):
                    res[six.text_type(filename)] = mf_file.readlines()
                else:
                    res[six.text_type(filename)] = \
                            [line for line in mf_file.readlines()
                                    if not line.startswith(b'#')]
            finally:
                mf_file.close()
        return res

    @mvn_depmap('JPP-bndlib.pom', 'usr/share/java/bndlib.jar')
    def test_basic(self, stdout, stderr, return_value, depmap):
        self.assertEqual(return_value, 0)
        report = self.check_result(inspect.currentframe().f_code.co_name,
                                           depmap)
        self.assertEqual(report, '', report)

    @mvn_depmap('JPP-commons-io.pom')
    def test_missing_jar_arg(self, stdout, stderr, return_value, depmap):
        self.assertNotEqual(return_value, 0)

    @mvn_depmap('JPP-apache-commons-io.pom')
    def test_packaging_pom_no_jar(self, stdout, stderr, return_value, depmap):
        self.assertEqual(return_value, 0, stderr)
        report = self.check_result(inspect.currentframe().f_code.co_name,
                                           depmap)
        self.assertEqual(report, '', report)

    @mvn_depmap('JPP-noversion.pom')
    def test_missing_version(self, stdout, stderr, return_value, depmap):
        self.assertNotEqual(return_value, 0)

    @mvn_depmap('JPP-parent-version.pom')
    def test_parent_version(self, stdout, stderr, return_value, depmap):
        self.assertEqual(return_value, 0, stderr)
        report = self.check_result(inspect.currentframe().f_code.co_name,
                                           depmap)
        self.assertEqual(report, '', report)

    @mvn_depmap('JPP-commons-war.pom', 'usr/share/java/commons-war.war')
    def test_war(self, stdout, stderr, return_value, depmap):
        self.assertEqual(return_value, 0, stderr)
        report = self.check_result(inspect.currentframe().f_code.co_name,
                                           depmap)
        self.assertEqual(report, '', report)

    def test_not_pom(self):
        stdout, stderr, return_value = call_script(os.path.join(DIRPATH, '..',
            'java-utils', 'maven_depmap.py'),
                ['.out', 'usr/share/java/commons-io.jar',
                    'usr/share/java/commons-io.jar'])
        self.assertNotEqual(return_value, 0)

    def test_no_pom(self):
        stdout, stderr, return_value = call_script(os.path.join(DIRPATH, '..',
            'java-utils', 'maven_depmap.py'),
                ['.out'])
        self.assertNotEqual(return_value, 0)

    def test_no_args(self):
        stdout, stderr, return_value = call_script(os.path.join(DIRPATH, '..',
            'java-utils', 'maven_depmap.py'),
                [])
        self.assertNotEqual(return_value, 0)

    @mvn_depmap('a:b:12', 'usr/share/java/commons-io.jar')
    def test_mvn_spec(self, stdout, stderr, return_value, depmap):
        self.assertEqual(return_value, 0, stderr)
        report = self.check_result(inspect.currentframe().f_code.co_name,
                                           depmap)
        self.assertEqual(report, '', report)

    @mvn_depmap('a:b:war::1', 'usr/share/java/commons-war.war')
    def test_mvn_spec_war(self, stdout, stderr, return_value, depmap):
        self.assertEqual(return_value, 0, stderr)
        report = self.check_result(inspect.currentframe().f_code.co_name,
                                           depmap)
        self.assertEqual(report, '', report)

    @mvn_depmap('/builddir/build/BUILDROOT/pkg-2.5.2-2.fc21.x86_64/a:b:war::1',
            'usr/share/java/commons-war.war')
    def test_mvn_spec_buildroot(self, stdout, stderr, return_value, depmap):
        self.assertEqual(return_value, 0, stderr)
        report = self.check_result(inspect.currentframe().f_code.co_name,
                                           depmap)
        self.assertEqual(report, '', report)

    @mvn_depmap('builddir/build/BUILDROOT/usr/share/maven-poms/JPP-commons-war.pom',
                'usr/share/java/commons-war.war')
    def test_buildroot(self, stdout, stderr, return_value, depmap):
        self.assertEqual(return_value, 0, stderr)
        report = self.check_result(inspect.currentframe().f_code.co_name,
                                   depmap)
        self.assertEqual(report, '', report)

    @mvn_depmap('a:b:12', 'usr/share/java/commons-io.jar', ['-a', 'x:y'])
    def test_append(self, stdout, stderr, return_value, depmap):
        self.assertEqual(return_value, 0, stderr)
        report = self.check_result(inspect.currentframe().f_code.co_name,
                                           depmap)
        self.assertEqual(report, '', report)

    # FIXME: aliases cause trouble
    #@mvn_depmap('a:b:12', 'usr/share/java/commons-io.jar', ['-a', 'x:y,z:w'])
    #def test_append_multiple(self, stdout, stderr, return_value, depmap):
    #    self.assertEqual(return_value, 0, stderr)
    #    report = self.check_result(inspect.currentframe().f_code.co_name,
    #                                       depmap)
    #    self.assertEqual(report, '', report)

    @mvn_depmap('a:b:12', 'usr/share/java/commons-io.jar', ['-n', 'myns'])
    def test_namespace(self, stdout, stderr, return_value, depmap):
        self.assertEqual(return_value, 0, stderr)
        report = self.check_result(inspect.currentframe().f_code.co_name,
                                           depmap)
        self.assertEqual(report, '', report)

    @mvn_depmap('a:b:12', 'usr/share/java/commons-io.jar', ['--namespace=myns',
        '--append=x:y'])
    def test_append_and_namespace(self, stdout, stderr, return_value, depmap):
        self.assertEqual(return_value, 0, stderr)
        report = self.check_result(inspect.currentframe().f_code.co_name,
                                           depmap)
        self.assertEqual(report, '', report)

    @mvn_depmap('a:b:12', 'usr/foo/share/java/compare_jar.jar')
    def test_compare_jar(self, stdout, stderr, return_value, depmap):
        self.assertEqual(return_value, 0, stderr)
        got, want = self.check_archive(inspect.currentframe().f_code.co_name,
                'usr/foo/share/java/compare_jar.jar')
        self.assertEqual(got, want)

    #test case for rhbz#1012982
    @mvn_depmap('x:y:war:z:0.1', 'usr/foo/share/java/compare_jar_class_ext.war')
    def test_compare_jar_class_ext(self, stdout, stderr, return_value, depmap):
        self.assertEqual(return_value, 0, stderr)
        got, want = self.check_archive(inspect.currentframe().f_code.co_name,
                'usr/foo/share/java/compare_jar_class_ext.war')
        self.assertEqual(got, want)

    @mvn_depmap('a:b:12', 'usr/share/java/already-has-pom-properties.jar')
    def test_compare_jar_unmodified(self, stdout, stderr, return_value, depmap):
        self.assertEqual(return_value, 0, stderr)
        got, want = self.check_archive(inspect.currentframe().f_code.co_name,
                'usr/share/java/already-has-pom-properties.jar', keep_comments=True)
        self.assertEqual(got, want)

    @mvn_depmap('x:y:0.1', 'usr/share/java/already-has-pom-properties.jar')
    def test_compare_jar_modified(self, stdout, stderr, return_value, depmap):
        self.assertEqual(return_value, 0, stderr)
        got, want = self.check_archive(inspect.currentframe().f_code.co_name,
                'usr/share/java/already-has-pom-properties.jar')
        self.assertEqual(got, want)

    @mvn_depmap('/builddir/build/BUILDROOT/pkg-2.5.2-2.fc21.x86_64/x:y:0.1',
            'usr/share/java/already-has-pom-properties.jar')
    def test_rhbz1012245(self, stdout, stderr, return_value, depmap):
        self.assertEqual(return_value, 0, stderr)
        got, want = self.check_archive('test_compare_jar_modified',
                'usr/share/java/already-has-pom-properties.jar')
        self.assertEqual(got, want)

    @mvn_depmap('x:y:jar:z:0.1', 'usr/share/java/commons-io-z.jar',
            ['-a', 'a:b:jar:c:12'])
    def test_classifier(self, stdout, stderr, return_value, depmap):
        self.assertEqual(return_value, 0, stderr)
        report = self.check_result(inspect.currentframe().f_code.co_name,
                                           depmap)
        self.assertEqual(report, '', report)

    @mvn_depmap('x:y:0.1', 'usr/share/java/commons-io.jar', ['-r', '1,2,3'])
    def test_version(self, stdout, stderr, return_value, depmap):
        self.assertEqual(return_value, 0, stderr)
        report = self.check_result(inspect.currentframe().f_code.co_name,
                                           depmap)
        self.assertEqual(report, '', report)

    @mvn_depmap('JPP/x:y:0.1', 'usr/share/java/commons-io.jar',
            ['--versions', '1,2,3'])
    def test_version2(self, stdout, stderr, return_value, depmap):
        self.assertEqual(return_value, 0, stderr)
        report = self.check_result('test_version', depmap)
        self.assertEqual(report, '', report)

    @mvn_depmap('x:y:0.1', 'usr/share/java/commons-io.jar',
            ['-r', '1,2,3', '-a', 'a:b:32'])
    def test_version_append(self, stdout, stderr, return_value, depmap):
        self.assertEqual(return_value, 0, stderr)
        report = self.check_result(inspect.currentframe().f_code.co_name,
                                           depmap)
        self.assertEqual(report, '', report)

    @mvn_depmap('x:y:0.1', 'usr/share/java/commons-io.jar',
            ['-r', '1,2,3', '-n', 'ns', '-a', 'a:b:32'])
    def test_version_namespace(self, stdout, stderr, return_value, depmap):
        self.assertEqual(return_value, 0, stderr)
        report = self.check_result(inspect.currentframe().f_code.co_name,
                                           depmap)
        self.assertEqual(report, '', report)

    @mvn_depmap('x:y', 'usr/share/java/commons-io.jar')
    def test_missing_version2(self, stdout, stderr, return_value, depmap):
        self.assertNotEqual(return_value, 0)

    @mvn_depmap('x:y:war:1', 'usr/share/java/commons-io.jar')
    def test_incorrect_extension(self, stdout, stderr, return_value, depmap):
        self.assertNotEqual(return_value, 0)

    @mvn_depmap('evil:', 'usr/share/java/commons-io.jar')
    def test_incorrect_artifact(self, stdout, stderr, return_value, depmap):
        self.assertNotEqual(return_value, 0)

    @mvn_depmap('JPP-cdata.pom')
    def test_cdata(self, stdout, stderr, return_value, depmap):
        self.assertEqual(return_value, 0, stderr)
        report = self.check_result(inspect.currentframe().f_code.co_name,
                                           depmap)
        self.assertEqual(report, '', report)

    @mvn_depmap('g:a:war:1.2.3', 'usr/share/java/versioned.war', ['-r', '2.0.0'])
    def test_versioned(self, stdout, stderr, return_value, depmap):
        self.assertEqual(return_value, 0, stderr)
        report = self.check_result(inspect.currentframe().f_code.co_name,
                                           depmap)
        self.assertEqual(report, '', report)
        self.assertEqual(False, os.path.exists('usr/share/java/versioned.war'))
        self.assertEqual(True, os.path.exists('usr/share/java/versioned-2.0.0.war'))

    @mvn_depmap('g:a:1.2', 'usr/share/java/versioned2.jar', ['-r', '1.2'])
    def test_versioned2(self, stdout, stderr, return_value, depmap):
        self.assertEqual(return_value, 0, stderr)
        report = self.check_result(inspect.currentframe().f_code.co_name,
                                           depmap)
        self.assertEqual(report, '', report)
        self.assertEqual(False, os.path.exists('usr/share/java/versioned2.jar'))
        self.assertEqual(True, os.path.exists('usr/share/java/versioned2-1.2.jar'))

    @mvn_depmap('g:a:jar:tests:1', 'usr/share/java/versioned-3-tests.jar', ['-r', '1,1.2'])
    def test_versioned_classifier(self, stdout, stderr, return_value, depmap):
        self.assertEqual(return_value, 0, stderr)
        report = self.check_result(inspect.currentframe().f_code.co_name,
                                           depmap)
        self.assertEqual(report, '', report)
        self.assertEqual(False, os.path.exists('usr/share/java/versioned-3-tests.jar'))
        self.assertEqual(True, os.path.exists('usr/share/java/versioned-3-tests-1.jar'))
        self.assertEqual(True, os.path.exists('usr/share/java/versioned-3-tests-1.2.jar'))

    @mvn_depmap('JPP-testversioned.pom', 'usr/share/java/testversioned.jar', ['-r', '2013.10'])
    def test_versioned_with_pom(self, stdout, stderr, return_value, depmap):
        self.assertEqual(return_value, 0, stderr)
        report = self.check_result(inspect.currentframe().f_code.co_name,
                                           depmap)
        self.assertEqual(report, '', report)
        self.assertEqual(False, os.path.exists('usr/share/java/testversioned.jar'))
        self.assertEqual(True, os.path.exists('usr/share/java/testversioned-2013.10.jar'))

    @mvn_depmap('JPP-alias.pom', 'usr/share/java/commons-io.jar', ['-a', 'a:b'])
    def test_alias_extension(self, stdout, stderr, return_value, depmap):
        self.assertEqual(return_value, 0, stderr)
        report = self.check_result(inspect.currentframe().f_code.co_name,
                                           depmap)
        self.assertEqual(report, '', report)

    def test_missing_jar(self):
        p = Package('test')
        p.append_to_prep("%add_maven_depmap g:a:1 this/file/doesnt/exist.jar")
        _, stderr, return_value = p.run_prep()
        self.assertEqual(1, return_value, 'bad return value')
        assertIn(self, 'file not found', stderr)

if __name__ == '__main__':
    unittest.main()



import os
import re
import shutil
import unittest

from xml_compare import compare_xml_files
from test_common import DIRPATH
from test_rpmbuild import Package

WORKDIR = os.path.join(DIRPATH, '.workdir')

unordered = ['artifactGlob', 'alias', 'compatVersions']


def rpm_test():
    def test_decorator(function):
        def test_decorated(self):
            pack = Package(function.__name__)
            function(self, pack)
        return test_decorated
    return test_decorator

class MvnMacrosTest(unittest.TestCase):
    maxDiff = 2048

    def setUp(self):
        self.olddir = os.getcwd()
        try:
            shutil.rmtree(WORKDIR)
        except OSError:
            pass
        try:
            os.mkdir(WORKDIR)
        except OSError:
            pass
        os.chdir(WORKDIR)

    def tearDown(self):
        try:
            shutil.rmtree(WORKDIR)
        except OSError:
            pass
        os.chdir(self.olddir)

    # FIXME: aliases cause trouble
    #@rpm_test()
    #def test_mvn_alias_simple(self, pack):
    #    pack.append_to_prep('%mvn_alias aaa:bbb xxx:yyy')
    #    _, stderr, return_value = pack.run_prep()
    #    self.assertEqual(return_value, 0, stderr)
    #    confpath = os.path.join(pack.buildpath, '.xmvn', 'config.d')
    #    filelist = os.listdir(confpath)
    #    self.assertEqual(len(filelist), 1)
    #    actfile = os.path.join(confpath, filelist[0])
    #    expfile = os.path.join(DIRPATH, 'data', 'mvn_alias', 'simple_00001.xml')
    #    report = compare_xml_files(actfile, expfile, unordered)
    #    self.assertEqual(report, '', report)

    @rpm_test()
    def test_mvn_alias_backref(self, pack):
        pack.append_to_prep("""%mvn_alias '*:{aaa,bbb}*' ':@1'""")
        _, stderr, return_value = pack.run_prep()
        self.assertEqual(return_value, 0, stderr)
        confpath = os.path.join(pack.buildpath, '.xmvn', 'config.d')
        filelist = os.listdir(confpath)
        self.assertEqual(len(filelist), 1)
        actfile = os.path.join(confpath, filelist[0])
        expfile = os.path.join(DIRPATH, 'data', 'mvn_alias',
                               'backref_00001.xml')
        report = compare_xml_files(actfile, expfile, unordered)
        self.assertEqual(report, '', report)

    #@rpm_test()
    #def test_mvn_alias_multi(self, pack):
    #    pack.append_to_prep('%mvn_alias "aaa:bbb" "ccc:ddd" \
    #                         "eee:fff" "ggg:hhh"')
    #    _, stderr, return_value = pack.run_prep()
    #    self.assertEqual(return_value, 0, stderr)
    #    confpath = os.path.join(pack.buildpath, '.xmvn', 'config.d')
    #    filelist = os.listdir(confpath)
    #    self.assertEqual(len(filelist), 1)
    #    actfile = os.path.join(confpath, filelist[0])
    #    expfile = os.path.join(DIRPATH, 'data', 'mvn_alias',
    #                           'multi_00001.xml')
    #    report = compare_xml_files(actfile, expfile, unordered)
    #    self.assertEqual(report, '', report)

    @rpm_test()
    def test_mvn_alias_no_args(self, pack):
        pack.append_to_prep('%mvn_alias')
        _, stderr, return_value = pack.run_prep()
        self.assertNotEqual(return_value, 0)
        self.assertNotEqual(stderr, '')

    @rpm_test()
    def test_mvn_alias_single_arg(self, pack):
        pack.append_to_prep('%mvn_alias "aaa:bbb"')
        _, stderr, return_value = pack.run_prep()
        self.assertNotEqual(return_value, 0)
        self.assertNotEqual(stderr, '')

    #@rpm_test()
    #def test_mvn_alias_more_invocations(self, pack):
    #    pack.append_to_prep('%mvn_alias "aaa:bbb" "ccc:ddd"')
    #    pack.append_to_prep('%mvn_alias "{xxx,yyy}:zzz" "qqq:@1"')
    #    _, stderr, return_value = pack.run_prep()
    #    self.assertEqual(return_value, 0, stderr)
    #    confpath = os.path.join(pack.buildpath, '.xmvn', 'config.d')
    #    filelist = sorted(os.listdir(confpath))
    #    self.assertEqual(len(filelist), 2)
    #    for i in range(2):
    #        actfile = os.path.join(confpath, filelist[i])
    #        filename = 'more_invocations-0000{i}.xml'.format(i=i + 1)
    #        expfile = os.path.join(DIRPATH, 'data', 'mvn_alias', filename)
    #        report = compare_xml_files(actfile, expfile, unordered)
    #        self.assertEqual(report, '', report)

    @rpm_test()
    def test_mvn_file_simple(self, pack):
        pack.append_to_prep('%mvn_file :x file')
        _, stderr, return_value = pack.run_prep()
        self.assertEqual(return_value, 0, stderr)
        confpath = os.path.join(pack.buildpath, '.xmvn', 'config.d')
        filelist = os.listdir(confpath)
        self.assertEqual(len(filelist), 1)
        actfile = os.path.join(confpath, filelist[0])
        expfile = os.path.join(DIRPATH, 'data', 'mvn_file', 'simple_00001.xml')
        report = compare_xml_files(actfile, expfile, unordered)
        self.assertEqual(report, '', report)

    @rpm_test()
    def test_mvn_file_symlinks(self, pack):
        pack.append_to_prep('%mvn_file :a file sym1 sym2 sym3')
        _, stderr, return_value = pack.run_prep()
        self.assertEqual(return_value, 0, stderr)
        confpath = os.path.join(pack.buildpath, '.xmvn', 'config.d')
        filelist = os.listdir(confpath)
        self.assertEqual(len(filelist), 1)
        actfile = os.path.join(confpath, filelist[0])
        expfile = os.path.join(DIRPATH, 'data', 'mvn_file',
                               'symlinks_00001.xml')
        report = compare_xml_files(actfile, expfile, unordered)
        self.assertEqual(report, '', report)

    @rpm_test()
    def test_mvn_file_no_args(self, pack):
        pack.append_to_prep('%mvn_file')
        _, stderr, return_value = pack.run_prep()
        self.assertNotEqual(return_value, 0)
        self.assertNotEqual(stderr, '')

    @rpm_test()
    def test_mvn_file_single_arg(self, pack):
        pack.append_to_prep('%mvn_file "aaa:bbb"')
        _, stderr, return_value = pack.run_prep()
        self.assertNotEqual(return_value, 0)
        self.assertNotEqual(stderr, '')

    @rpm_test()
    def test_mvn_compat_version_simple(self, pack):
        pack.append_to_prep('%mvn_compat_version aaa:bbb 1')
        _, stderr, return_value = pack.run_prep()
        self.assertEqual(return_value, 0, stderr)
        confpath = os.path.join(pack.buildpath, '.xmvn', 'config.d')
        filelist = os.listdir(confpath)
        self.assertEqual(len(filelist), 1)
        actfile = os.path.join(confpath, filelist[0])
        expfile = os.path.join(DIRPATH, 'data', 'mvn_compat_version',
                               'simple_00001.xml')
        report = compare_xml_files(actfile, expfile, unordered)
        self.assertEqual(report, '', report)

    @rpm_test()
    def test_mvn_compat_version_no_args(self, pack):
        pack.append_to_prep('%mvn_compat_version')
        _, stderr, return_value = pack.run_prep()
        self.assertNotEqual(return_value, 0)
        self.assertNotEqual(stderr, '')

    @rpm_test()
    def test_mvn_compat_version_single(self, pack):
        pack.append_to_prep('%mvn_compat_version "aaa:bbb"')
        _, stderr, return_value = pack.run_prep()
        self.assertNotEqual(return_value, 0)
        self.assertNotEqual(stderr, '')

    @rpm_test()
    def test_mvn_package_simple(self, pack):
        pack.append_to_prep('%mvn_package aaa:bbb pack')
        _, stderr, return_value = pack.run_prep()
        self.assertEqual(return_value, 0, stderr)
        confpath = os.path.join(pack.buildpath, '.xmvn', 'config.d')
        filelist = os.listdir(confpath)
        self.assertEqual(len(filelist), 1)
        actfile = os.path.join(confpath, filelist[0])
        expfile = os.path.join(DIRPATH, 'data', 'mvn_package',
                               'simple_00001.xml')
        report = compare_xml_files(actfile, expfile, unordered)
        self.assertEqual(report, '', report)

    @rpm_test()
    def test_mvn_package_no_args(self, pack):
        pack.append_to_prep('%mvn_package')
        _, stderr, return_value = pack.run_prep()
        self.assertNotEqual(return_value, 0)
        self.assertNotEqual(stderr, '')

    @rpm_test()
    def test_mvn_package_single(self, pack):
        pack.append_to_prep('%mvn_package aaa:bbb')
        _, stderr, return_value = pack.run_prep()
        self.assertEqual(return_value, 0, stderr)
        confpath = os.path.join(pack.buildpath, '.xmvn', 'config.d')
        filelist = os.listdir(confpath)
        self.assertEqual(len(filelist), 1)
        actfile = os.path.join(confpath, filelist[0])
        expfile = os.path.join(DIRPATH, 'data', 'mvn_package',
                               'single_00001.xml')
        report = compare_xml_files(actfile, expfile, unordered)
        self.assertEqual(report, '', report)

    @rpm_test()
    def test_mvn_config_no_args(self, pack):
        pack.append_to_prep('%mvn_config')
        _, stderr, return_value = pack.run_prep()
        self.assertNotEqual(return_value, 0)
        self.assertNotEqual(stderr, '')

    @rpm_test()
    def test_mvn_config_single(self, pack):
        pack.append_to_prep('%mvn_config')
        _, stderr, return_value = pack.run_prep()
        self.assertNotEqual(return_value, 0)
        self.assertNotEqual(stderr, '')

    @rpm_test()
    def test_mvn_config_path(self, pack):
        pack.append_to_prep('%mvn_config a/b/c xxx')
        _, stderr, return_value = pack.run_prep()
        self.assertEqual(return_value, 0, stderr)
        confpath = os.path.join(pack.buildpath, '.xmvn', 'config.d')
        filelist = os.listdir(confpath)
        self.assertEqual(len(filelist), 1)
        actfile = os.path.join(confpath, filelist[0])
        expfile = os.path.join(DIRPATH, 'data', 'mvn_config',
                               'path_00001.xml')
        report = compare_xml_files(actfile, expfile, unordered)
        self.assertEqual(report, '', report)

    @rpm_test()
    def test_mvn_config_multiline(self, pack):
        pack.append_to_prep("""%mvn_config a/b/c "
            <bla>
                bla
            </bla>
        " """)
        _, stderr, return_value = pack.run_prep()
        self.assertEqual(return_value, 0, stderr)
        confpath = os.path.join(pack.buildpath, '.xmvn', 'config.d')
        filelist = os.listdir(confpath)
        self.assertEqual(len(filelist), 1)
        actfile = os.path.join(confpath, filelist[0])
        expfile = os.path.join(DIRPATH, 'data', 'mvn_config',
                               'multiline_00001.xml')
        report = compare_xml_files(actfile, expfile, unordered)
        self.assertEqual(report, '', report)

    @rpm_test()
    def test_mvn_build_no_args(self, pack):
        pack.append_to_build('%mvn_build')

        _, stderr, return_value = pack.run_build()
        self.assertEqual(return_value, 0, stderr)
        argspath = os.path.join(pack.buildpath, '.xmvn', 'out')
        with open(argspath, 'r') as argsfile:
            exp = '--batch-mode --offline ' + \
                  'verify org.fedoraproject.xmvn:xmvn-mojo:install ' + \
                  'org.apache.maven.plugins:maven-javadoc-plugin:aggregate ' + \
                  'org.fedoraproject.xmvn:xmvn-mojo:builddep\n'
            self.assertEqual(argsfile.read(), exp)

    @rpm_test()
    def test_mvn_build_singleton(self, pack):
        pack.append_to_build('%mvn_build -s')
        _, stderr, return_value = pack.run_build()
        self.assertEqual(return_value, 0, stderr)
        confpath = os.path.join(pack.buildpath, '.xmvn', 'config.d')
        filelist = sorted(os.listdir(confpath))
        self.assertEqual(len(filelist), 2)
        for i in range(2):
            actfile = os.path.join(confpath, filelist[i])
            filename = 'singleton_0000{i}.xml'.format(i=i + 1)
            expfile = os.path.join(DIRPATH, 'data', 'mvn_build', filename)
            report = compare_xml_files(actfile, expfile, unordered)
            self.assertEqual(report, '', report)

        expfile = os.path.join(pack.buildpath, '.xmvn', 'out')
        actfile = os.path.join(DIRPATH, 'data', 'mvn_build',
                                   'singleton_out')
        with open(actfile, 'r') as act:
            with open(expfile, 'r') as exp:
                self.assertEqual(act.read(), exp.read())

    @rpm_test()
    def test_mvn_build_environment(self, pack):
        pack.append_to_build('export MAVEN_OPTS="-Xmx1024M"')
        pack.append_to_build('%mvn_build')
        _, stderr, return_value = pack.run_build()
        self.assertEqual(return_value, 0, stderr)
        envpath = os.path.join(pack.buildpath, '.xmvn', 'env')
        with open(envpath, 'r') as envfile:
            for line in envfile:
                if line == 'MAVEN_OPTS=-Xmx1024M\n':
                    break
            else:
                self.assertTrue(False, 'Environment varible not passed to XMvn')

    @rpm_test()
    def test_mvn_build_pass_args(self, pack):
        pack.append_to_build('%mvn_build -s -- --log-file "/var/log/xmvn"')
        _, stderr, return_value = pack.run_build()
        self.assertEqual(return_value, 0, stderr)
        argspath = os.path.join(pack.buildpath, '.xmvn', 'out')
        with open(argspath, 'r') as argsfile:
            exp = '--batch-mode --offline --log-file /var/log/xmvn ' + \
                  'verify org.fedoraproject.xmvn:xmvn-mojo:install ' + \
                  'org.apache.maven.plugins:maven-javadoc-plugin:aggregate ' + \
                  'org.fedoraproject.xmvn:xmvn-mojo:builddep\n'
            self.assertEqual(argsfile.read(), exp)

    @rpm_test()
    def test_mvn_build_bootstrap(self, pack):
        pack.append_to_build('%define xmvn_bootstrap true')
        pack.append_to_build('%mvn_build')

        _, stderr, return_value = pack.run_build()
        self.assertEqual(return_value, 0, stderr)
        argspath = os.path.join(pack.buildpath, '.xmvn', 'out')
        with open(argspath, 'r') as argsfile:
            exp = '--batch-mode ' + \
                  'verify org.fedoraproject.xmvn:xmvn-mojo:install ' + \
                  'org.apache.maven.plugins:maven-javadoc-plugin:aggregate ' + \
                  'org.fedoraproject.xmvn:xmvn-mojo:builddep\n'
            self.assertEqual(argsfile.read(), exp)

    @rpm_test()
    def test_mvn_build_skip_javadoc(self, pack):
        pack.prepend('%bcond_without javadoc')
        pack.append_to_build('%mvn_build')

        _, stderr, return_value = pack.run_build(['--without', 'javadoc'])
        self.assertEqual(return_value, 0, stderr)
        argspath = os.path.join(pack.buildpath, '.xmvn', 'out')
        with open(argspath, 'r') as argsfile:
            exp = '--batch-mode --offline ' + \
                  'verify org.fedoraproject.xmvn:xmvn-mojo:install ' + \
                  'org.fedoraproject.xmvn:xmvn-mojo:builddep\n'
            self.assertEqual(argsfile.read(), exp)

    @rpm_test()
    def test_mvn_build_skip_test(self, pack):
        pack.prepend('%bcond_without tests')
        pack.append_to_build('%mvn_build')

        _, stderr, return_value = pack.run_build(['--without', 'tests'])
        self.assertEqual(return_value, 0, stderr)
        argspath = os.path.join(pack.buildpath, '.xmvn', 'out')
        with open(argspath, 'r') as argsfile:
            exp = '--batch-mode --offline -Dmaven.test.skip=true ' + \
                  'package org.fedoraproject.xmvn:xmvn-mojo:install ' + \
                  'org.apache.maven.plugins:maven-javadoc-plugin:aggregate ' + \
                  'org.fedoraproject.xmvn:xmvn-mojo:builddep\n'
            self.assertEqual(argsfile.read(), exp)

    @rpm_test()
    def test_mvn_install(self, pack):
        pack.append_to_install('%mvn_install')

        _, stderr, return_value = pack.run_install()
        self.assertEqual(return_value, 0, stderr)

        argspath = os.path.join('rpmbuild', 'BUILD', '.xmvn', 'install-out')
        with open(argspath, 'r') as argsfile:
            args = argsfile.read()
            self.assertTrue(re.match(r'-R \.xmvn-reactor -n \S+ -d \S+\n',
                                     args), args)

    @rpm_test()
    def test_mvn_install_debug(self, pack):
        pack.append_to_install('%mvn_install -X')

        _, stderr, return_value = pack.run_install()
        self.assertEqual(return_value, 0, stderr)

        argspath = os.path.join('rpmbuild', 'BUILD', '.xmvn', 'install-out')
        with open(argspath, 'r') as argsfile:
            args = argsfile.read()
            self.assertTrue(re.match(r'-X -R \.xmvn-reactor -n \S+ -d \S+\n',
                                     args), args)

    @rpm_test()
    def test_mvn_install_directory(self, pack):
        pack.append_to_build('mkdir ../doc')
        pack.append_to_build('touch ../doc/file')
        pack.append_to_install('%mvn_install -J doc')
        pack.append_to_install('%files -f .mfiles-javadoc')

        _, stderr, return_value = pack.run_install()
        self.assertEqual(return_value, 0, stderr)

        argspath = os.path.join('rpmbuild', 'BUILD', '.xmvn', 'install-out')
        with open(argspath, 'r') as argsfile:
            args = argsfile.read()
            self.assertTrue(re.match(r'-R \.xmvn-reactor -n \S+ -d \S+\n',
                                     args), args)

        mfilespath = os.path.join('rpmbuild', 'BUILD', '.mfiles-javadoc')
        with open(mfilespath, 'r') as mfiles:
            self.assertEqual(mfiles.read(),
                             '/usr/share/javadoc/test_mvn_install_directory\n')

if __name__ == '__main__':
    unittest.main()



#
# Copyright (c) 2015, Red Hat, Inc.
#
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
#
# 1. Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in the
#    documentation and/or other materials provided with the
#    distribution.
# 3. Neither the name of the Red Hat nor the names of its
#    contributors may be used to endorse or promote products derived
#    from this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
# A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
# OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
# LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
# DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
# THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#
# Authors:  Michal Srb <msrb@redhat.com>

from javapackages.maven.artifact import Artifact
import javapackages.common.strutils as Printer

from javapackages.common.binding import ObjectBinding


class MetadataSkippedArtifact(ObjectBinding):
    element_name = 'skippedArtifact'
    fields = ['groupId', 'artifactId', 'extension', 'classifier']
    defaults = {'extension': 'jar'}

    def get_mvn_str(self):
        return Printer.get_mvn_str(self.groupId,
                                   self.artifactId,
                                   ext=self.extension,
                                   cla=self.classifier)

    @classmethod
    def from_mvn_str(cls, mvn_str):
        a = Artifact.from_mvn_str(mvn_str)

        return cls(a.groupId, a.artifactId, extension=a.extension,
                   classifier=a.classifier)
